
 MODULE mod_chem_spack_jacdchemdc

   USE mod_chem_spack_dratedc, ONLY: dratedc  ! subroutine
   IMPLICIT NONE
   PRIVATE
   PUBLIC :: jacdchemdc ! subroutine
 CONTAINS

   SUBROUTINE jacdchemdc(y,rk,JacC,ngas,ijkbeg,ijkend,maxblock_size,nr)

!------------------------------------------------------------------------
!
!     -- DESCRIPTION
!
!     This routine computes the Jacobian matrix for the gas-phase.
!     This routine is automatically generated by SPACK.
!     Mechanism: ../Mechanism/RELACS
!     Species: ../Mechanism/ciRLCS
!
!------------------------------------------------------------------------
!
!     -- INPUT VARIABLES
!
!     Y: chemical concentrations.
!     RK: kinetic rates.
!
!     -- INPUT/OUTPUT VARIABLES
!
!     -- OUTPUT VARIABLES
!
!     JACC: Jacobian matrix.
!
!------------------------------------------------------------------------
!
!     -- REMARKS
!
!     The matrix JACC could be stored in a low-dimensional vector.
!
!------------------------------------------------------------------------
!
!     -- MODIFICATIONS
!
!------------------------------------------------------------------------
!
!     -- AUTHOR(S)
!
!     SPACK.
!
!------------------------------------------------------------------------

      IMPLICIT NONE


       INTEGER 	 , INTENT(IN)  :: ngas
       INTEGER 	 , INTENT(IN)  :: ijkbeg
       INTEGER 	 , INTENT(IN)  :: ijkend
       INTEGER 	 , INTENT(IN)  :: maxblock_size
       INTEGER 	 , INTENT(IN)  :: nr
       DOUBLE PRECISION , INTENT(IN)  :: rk(maxblock_size,nr)
       DOUBLE PRECISION , INTENT(IN)  :: y(maxblock_size,NGAS)
       DOUBLE PRECISION , INTENT(OUT) :: JacC(maxblock_size,NGAS,NGAS)

       DOUBLE PRECISION :: dw(maxblock_size,nr,NGAS)
       INTEGER :: ijk


      CALL dratedc(rk,y,dw,ngas,ijkbeg,ijkend,maxblock_size,nr)

      !DO ijk=ijkbeg,ijkend
      JacC(ijkbeg:ijkend,  3,  4) =  + dw(ijkbeg:ijkend,  1,  4) &
                           + dw(ijkbeg:ijkend, 30,  4) &
                           + dw(ijkbeg:ijkend, 46,  4)
      JacC(ijkbeg:ijkend,  4,  4) =  - dw(ijkbeg:ijkend,  1,  4) &
                           - dw(ijkbeg:ijkend, 30,  4) &
                           - dw(ijkbeg:ijkend, 31,  4) &
                           - dw(ijkbeg:ijkend, 33,  4) &
                           - dw(ijkbeg:ijkend, 36,  4) &
                           - dw(ijkbeg:ijkend, 43,  4) &
                           - dw(ijkbeg:ijkend, 47,  4) &
                           - dw(ijkbeg:ijkend, 80,  4) &
                           - dw(ijkbeg:ijkend, 82,  4) &
                           - dw(ijkbeg:ijkend, 85,  4)
      JacC(ijkbeg:ijkend, 18,  4) =  + dw(ijkbeg:ijkend,  1,  4) &
                           - dw(ijkbeg:ijkend, 30,  4) &
                           - dw(ijkbeg:ijkend, 31,  4)
      JacC(ijkbeg:ijkend,  1,  1) =  - dw(ijkbeg:ijkend,  2,  1) &
                           - dw(ijkbeg:ijkend,  3,  1) &
                           - dw(ijkbeg:ijkend, 19,  1) &
                           - dw(ijkbeg:ijkend, 23,  1) &
                           - dw(ijkbeg:ijkend, 24,  1) &
                           - dw(ijkbeg:ijkend, 42,  1) &
                           - dw(ijkbeg:ijkend, 43,  1) &
                           - dw(ijkbeg:ijkend, 76,  1) &
                           - dw(ijkbeg:ijkend, 77,  1) &
                           - dw(ijkbeg:ijkend, 78,  1) &
                           - dw(ijkbeg:ijkend, 79,  1) &
                           - dw(ijkbeg:ijkend, 84,  1)
      JacC(ijkbeg:ijkend, 15,  1) =  + dw(ijkbeg:ijkend,  2,  1) &
                           + dw(ijkbeg:ijkend,  3,  1)
      JacC(ijkbeg:ijkend, 19,  1) =  + dw(ijkbeg:ijkend,  2,  1)
      JacC(ijkbeg:ijkend, 18,  1) =  + dw(ijkbeg:ijkend,  3,  1) &
                           - dw(ijkbeg:ijkend, 19,  1) &
                          + 0.9000000000000000D-01*dw(ijkbeg:ijkend, 77,  1)
      JacC(ijkbeg:ijkend,  3,  7) =  + dw(ijkbeg:ijkend,  4,  7)
      JacC(ijkbeg:ijkend,  7,  7) =  - dw(ijkbeg:ijkend,  4,  7) &
                           - dw(ijkbeg:ijkend, 39,  7)
      JacC(ijkbeg:ijkend, 20,  7) =  + dw(ijkbeg:ijkend,  4,  7) &
                           - dw(ijkbeg:ijkend, 39,  7)
      JacC(ijkbeg:ijkend,  4,  8) =  + dw(ijkbeg:ijkend,  5,  8)
      JacC(ijkbeg:ijkend,  8,  8) =  - dw(ijkbeg:ijkend,  5,  8) &
                           - dw(ijkbeg:ijkend, 40,  8)
      JacC(ijkbeg:ijkend, 20,  8) =  + dw(ijkbeg:ijkend,  5,  8) &
                           - dw(ijkbeg:ijkend, 40,  8)
      JacC(ijkbeg:ijkend,  4,  9) = + 0.6500000000000000D+00*dw(ijkbeg:ijkend,  6,  9) &
                           + dw(ijkbeg:ijkend, 37,  9) &
                           + dw(ijkbeg:ijkend, 41,  9)
      JacC(ijkbeg:ijkend,  5,  9) = + 0.3500000000000000D+00*dw(ijkbeg:ijkend,  6,  9)
      JacC(ijkbeg:ijkend,  9,  9) =  - dw(ijkbeg:ijkend,  6,  9) &
                           - dw(ijkbeg:ijkend, 37,  9) &
                           - dw(ijkbeg:ijkend, 41,  9)
      JacC(ijkbeg:ijkend, 20,  9) = + 0.3500000000000000D+00*dw(ijkbeg:ijkend,  6,  9) &
                           - dw(ijkbeg:ijkend, 41,  9)
      JacC(ijkbeg:ijkend, 21,  9) = + 0.6500000000000000D+00*dw(ijkbeg:ijkend,  6,  9) &
                           + dw(ijkbeg:ijkend, 37,  9)
      JacC(ijkbeg:ijkend,  3,  5) =  + dw(ijkbeg:ijkend,  7,  5) &
                           - dw(ijkbeg:ijkend, 45,  5) &
                           + dw(ijkbeg:ijkend, 46,  5)
      JacC(ijkbeg:ijkend,  5,  5) =  - dw(ijkbeg:ijkend,  7,  5) &
                           - dw(ijkbeg:ijkend,  8,  5) &
                           - dw(ijkbeg:ijkend, 34,  5) &
                           - dw(ijkbeg:ijkend, 38,  5) &
                           - dw(ijkbeg:ijkend, 45,  5) &
                           - dw(ijkbeg:ijkend, 46,  5) &
                           - dw(ijkbeg:ijkend, 47,  5) &
                          - 0.2000000000000000D+01*dw(ijkbeg:ijkend, 49,  5) &
                          - 0.2000000000000000D+01*dw(ijkbeg:ijkend, 49,  5) &
                           - dw(ijkbeg:ijkend, 69,  5) &
                           - dw(ijkbeg:ijkend, 70,  5) &
                           - dw(ijkbeg:ijkend, 71,  5) &
                           - dw(ijkbeg:ijkend, 72,  5) &
                           - dw(ijkbeg:ijkend, 73,  5) &
                           - dw(ijkbeg:ijkend, 74,  5) &
                          - 0.4000000000000000D+00*dw(ijkbeg:ijkend, 75,  5) &
                           - dw(ijkbeg:ijkend,116,  5) &
                           - dw(ijkbeg:ijkend,117,  5) &
                           - dw(ijkbeg:ijkend,118,  5) &
                           - dw(ijkbeg:ijkend,119,  5) &
                           - dw(ijkbeg:ijkend,120,  5) &
                           - dw(ijkbeg:ijkend,121,  5) &
                           - dw(ijkbeg:ijkend,122,  5) &
                           - dw(ijkbeg:ijkend,128,  5)
      JacC(ijkbeg:ijkend, 15,  5) =  + dw(ijkbeg:ijkend,  7,  5)
      JacC(ijkbeg:ijkend,  4,  5) =  + dw(ijkbeg:ijkend,  8,  5) &
                           + dw(ijkbeg:ijkend, 34,  5) &
                          + 0.7000000000000000D+00*dw(ijkbeg:ijkend, 38,  5) &
                          + 0.2000000000000000D+01*dw(ijkbeg:ijkend, 45,  5) &
                           - dw(ijkbeg:ijkend, 47,  5) &
                          + 0.2000000000000000D+01*dw(ijkbeg:ijkend, 49,  5) &
                          + 0.2000000000000000D+01*dw(ijkbeg:ijkend, 49,  5) &
                          + 0.1053000000000000D+00*dw(ijkbeg:ijkend, 71,  5) &
                          + 0.4000000000000000D+00*dw(ijkbeg:ijkend, 75,  5) &
                           + dw(ijkbeg:ijkend,116,  5) &
                           + dw(ijkbeg:ijkend,117,  5) &
                           + dw(ijkbeg:ijkend,118,  5) &
                           + dw(ijkbeg:ijkend,119,  5) &
                           + dw(ijkbeg:ijkend,120,  5) &
                           + dw(ijkbeg:ijkend,121,  5) &
                          + 0.1740720000000000D+01*dw(ijkbeg:ijkend,122,  5) &
                           + dw(ijkbeg:ijkend,128,  5)
      JacC(ijkbeg:ijkend, 18,  5) =  + dw(ijkbeg:ijkend,  8,  5)
      JacC(ijkbeg:ijkend,  2,  2) =  - dw(ijkbeg:ijkend,  9,  2) &
                           - dw(ijkbeg:ijkend, 26,  2)
      JacC(ijkbeg:ijkend, 20,  2) = + 0.2000000000000000D+01*dw(ijkbeg:ijkend,  9,  2) &
                           - dw(ijkbeg:ijkend, 26,  2)
      JacC(ijkbeg:ijkend, 12, 28) =  + dw(ijkbeg:ijkend, 10, 28) &
                           + dw(ijkbeg:ijkend, 11, 28) &
                           + dw(ijkbeg:ijkend, 61, 28) &
                           + dw(ijkbeg:ijkend, 69, 28)
      JacC(ijkbeg:ijkend, 17, 28) =  + dw(ijkbeg:ijkend, 10, 28)
      JacC(ijkbeg:ijkend, 28, 28) =  - dw(ijkbeg:ijkend, 10, 28) &
                           - dw(ijkbeg:ijkend, 11, 28) &
                           - dw(ijkbeg:ijkend, 61, 28) &
                           - dw(ijkbeg:ijkend, 69, 28)
      JacC(ijkbeg:ijkend, 21, 28) = + 0.2000000000000000D+01*dw(ijkbeg:ijkend, 11, 28) &
                           + dw(ijkbeg:ijkend, 61, 28) &
                           + dw(ijkbeg:ijkend, 69, 28)
      JacC(ijkbeg:ijkend, 12, 29) =  + dw(ijkbeg:ijkend, 12, 29)
      JacC(ijkbeg:ijkend, 21, 29) =  + dw(ijkbeg:ijkend, 12, 29)
      JacC(ijkbeg:ijkend, 29, 29) =  - dw(ijkbeg:ijkend, 12, 29) &
                           - dw(ijkbeg:ijkend, 62, 29) &
                           - dw(ijkbeg:ijkend, 70, 29)
      JacC(ijkbeg:ijkend, 38, 29) =  + dw(ijkbeg:ijkend, 12, 29)
      JacC(ijkbeg:ijkend, 20, 34) =  + dw(ijkbeg:ijkend, 13, 34) &
                          - 0.6500000000000000D+00*dw(ijkbeg:ijkend, 65, 34)
      JacC(ijkbeg:ijkend, 21, 34) =  + dw(ijkbeg:ijkend, 13, 34)
      JacC(ijkbeg:ijkend, 28, 34) =  + dw(ijkbeg:ijkend, 13, 34) &
                          + 0.3500000000000000D+00*dw(ijkbeg:ijkend, 65, 34)
      JacC(ijkbeg:ijkend, 34, 34) =  - dw(ijkbeg:ijkend, 13, 34) &
                           - dw(ijkbeg:ijkend, 65, 34)
      JacC(ijkbeg:ijkend, 20, 35) =  + dw(ijkbeg:ijkend, 14, 35) &
                          - 0.5507500000000001D+00*dw(ijkbeg:ijkend, 66, 35)
      JacC(ijkbeg:ijkend, 21, 35) = + 0.9620500000000000D+00*dw(ijkbeg:ijkend, 14, 35) &
                          + 0.2915000000000000D-01*dw(ijkbeg:ijkend, 66, 35)
      JacC(ijkbeg:ijkend, 29, 35) = + 0.9620500000000000D+00*dw(ijkbeg:ijkend, 14, 35) &
                          + 0.7335000000000000D-01*dw(ijkbeg:ijkend, 66, 35)
      JacC(ijkbeg:ijkend, 35, 35) =  - dw(ijkbeg:ijkend, 14, 35) &
                           - dw(ijkbeg:ijkend, 66, 35)
      JacC(ijkbeg:ijkend, 38, 35) = + 0.3795000000000000D-01*dw(ijkbeg:ijkend, 14, 35)
      JacC(ijkbeg:ijkend, 30, 30) =  - dw(ijkbeg:ijkend, 15, 30) &
                           - dw(ijkbeg:ijkend, 63, 30)
      JacC(ijkbeg:ijkend, 39, 30) =  + dw(ijkbeg:ijkend, 15, 30)
      JacC(ijkbeg:ijkend, 45, 30) =  + dw(ijkbeg:ijkend, 15, 30) &
                           + dw(ijkbeg:ijkend, 63, 30)
      JacC(ijkbeg:ijkend, 12, 31) = + 0.9192399999999999D+00*dw(ijkbeg:ijkend, 16, 31) &
                          + 0.1017320000000000D+01*dw(ijkbeg:ijkend, 64, 31) &
                          + 0.1337230000000000D+01*dw(ijkbeg:ijkend, 71, 31) &
                          + 0.6472800000000000D+00*dw(ijkbeg:ijkend, 78, 31)
      JacC(ijkbeg:ijkend, 17, 31) = + 0.2084200000000000D+00*dw(ijkbeg:ijkend, 16, 31)
      JacC(ijkbeg:ijkend, 21, 31) = + 0.7583000000000000D+00*dw(ijkbeg:ijkend, 16, 31) &
                          + 0.5120800000000000D+00*dw(ijkbeg:ijkend, 64, 31) &
                          + 0.6321700000000000D+00*dw(ijkbeg:ijkend, 71, 31) &
                          + 0.2844100000000000D+00*dw(ijkbeg:ijkend, 78, 31)
      JacC(ijkbeg:ijkend, 28, 31) = + 0.6517000000000001D-01*dw(ijkbeg:ijkend, 16, 31)
      JacC(ijkbeg:ijkend, 31, 31) =  - dw(ijkbeg:ijkend, 16, 31) &
                           - dw(ijkbeg:ijkend, 54, 31) &
                          - 0.8308100000000000D+00*dw(ijkbeg:ijkend, 64, 31) &
                          - 0.8947000000000001D+00*dw(ijkbeg:ijkend, 71, 31) &
                          + 0.7583000000000006D-01*dw(ijkbeg:ijkend, 78, 31)
      JacC(ijkbeg:ijkend, 45, 31) = + 0.6962200000000000D+00*dw(ijkbeg:ijkend, 16, 31) &
                          + 0.5141900000000000D+00*dw(ijkbeg:ijkend, 64, 31) &
                          + 0.3888100000000000D+00*dw(ijkbeg:ijkend, 71, 31) &
                          + 0.2746000000000000D+00*dw(ijkbeg:ijkend, 78, 31)
      JacC(ijkbeg:ijkend,  4, 32) =  + dw(ijkbeg:ijkend, 17, 32) &
                           + dw(ijkbeg:ijkend, 68, 32)
      JacC(ijkbeg:ijkend, 21, 32) =  + dw(ijkbeg:ijkend, 17, 32)
      JacC(ijkbeg:ijkend, 29, 32) = + 0.2000000000000000D+00*dw(ijkbeg:ijkend, 17, 32)
      JacC(ijkbeg:ijkend, 30, 32) = + 0.8000000000000000D+00*dw(ijkbeg:ijkend, 17, 32)
      JacC(ijkbeg:ijkend, 32, 32) =  - dw(ijkbeg:ijkend, 17, 32) &
                           - dw(ijkbeg:ijkend, 68, 32)
      JacC(ijkbeg:ijkend,  1, 18) =  + dw(ijkbeg:ijkend, 18, 18) &
                           - dw(ijkbeg:ijkend, 19, 18)
      JacC(ijkbeg:ijkend, 18, 18) =  - dw(ijkbeg:ijkend, 18, 18) &
                           - dw(ijkbeg:ijkend, 19, 18) &
                           - dw(ijkbeg:ijkend, 29, 18) &
                           - dw(ijkbeg:ijkend, 30, 18) &
                           - dw(ijkbeg:ijkend, 31, 18) &
                           - dw(ijkbeg:ijkend, 53, 18) &
                           - dw(ijkbeg:ijkend, 54, 18)
      JacC(ijkbeg:ijkend, 18, 19) =  + dw(ijkbeg:ijkend, 20, 19) &
                           + dw(ijkbeg:ijkend, 21, 19)
      JacC(ijkbeg:ijkend, 19, 19) =  - dw(ijkbeg:ijkend, 20, 19) &
                           - dw(ijkbeg:ijkend, 21, 19) &
                           - dw(ijkbeg:ijkend, 22, 19)
      JacC(ijkbeg:ijkend, 20, 19) = + 0.2000000000000000D+01*dw(ijkbeg:ijkend, 22, 19)
      JacC(ijkbeg:ijkend,  1, 20) =  - dw(ijkbeg:ijkend, 23, 20)
      JacC(ijkbeg:ijkend, 20,  1) =  - dw(ijkbeg:ijkend, 23,  1) &
                           + dw(ijkbeg:ijkend, 24,  1) &
                          + 0.3943500000000000D+00*dw(ijkbeg:ijkend, 76,  1) &
                          + 0.2800000000000000D+00*dw(ijkbeg:ijkend, 77,  1) &
                          + 0.2059500000000000D+00*dw(ijkbeg:ijkend, 78,  1) &
                          + 0.3600000000000000D-01*dw(ijkbeg:ijkend, 79,  1) &
                           + dw(ijkbeg:ijkend, 84,  1)
      JacC(ijkbeg:ijkend, 20, 20) =  - dw(ijkbeg:ijkend, 23, 20) &
                           - dw(ijkbeg:ijkend, 25, 20) &
                           - dw(ijkbeg:ijkend, 26, 20) &
                           - dw(ijkbeg:ijkend, 32, 20) &
                           - dw(ijkbeg:ijkend, 33, 20) &
                           - dw(ijkbeg:ijkend, 34, 20) &
                           - dw(ijkbeg:ijkend, 39, 20) &
                           - dw(ijkbeg:ijkend, 40, 20) &
                           - dw(ijkbeg:ijkend, 41, 20) &
                           - dw(ijkbeg:ijkend, 50, 20) &
                           - dw(ijkbeg:ijkend, 51, 20) &
                           - dw(ijkbeg:ijkend, 52, 20) &
                           - dw(ijkbeg:ijkend, 55, 20) &
                           - dw(ijkbeg:ijkend, 56, 20) &
                          - 0.9991220000000000D+00*dw(ijkbeg:ijkend, 57, 20) &
                           - dw(ijkbeg:ijkend, 58, 20) &
                           - dw(ijkbeg:ijkend, 59, 20) &
                           - dw(ijkbeg:ijkend, 60, 20) &
                           - dw(ijkbeg:ijkend, 61, 20) &
                           - dw(ijkbeg:ijkend, 62, 20) &
                           - dw(ijkbeg:ijkend, 63, 20) &
                           - dw(ijkbeg:ijkend, 64, 20) &
                          - 0.6500000000000000D+00*dw(ijkbeg:ijkend, 65, 20) &
                          - 0.5507500000000001D+00*dw(ijkbeg:ijkend, 66, 20) &
                           - dw(ijkbeg:ijkend, 67, 20) &
                           - dw(ijkbeg:ijkend, 68, 20)
      JacC(ijkbeg:ijkend, 21,  1) =  + dw(ijkbeg:ijkend, 23,  1) &
                           - dw(ijkbeg:ijkend, 24,  1) &
                          + 0.2345100000000000D+00*dw(ijkbeg:ijkend, 76,  1) &
                          + 0.3000000000000000D+00*dw(ijkbeg:ijkend, 77,  1) &
                          + 0.2844100000000000D+00*dw(ijkbeg:ijkend, 78,  1) &
                          + 0.8000000000000000D-01*dw(ijkbeg:ijkend, 79,  1)
      JacC(ijkbeg:ijkend, 21, 20) =  + dw(ijkbeg:ijkend, 23, 20) &
                           - dw(ijkbeg:ijkend, 25, 20) &
                           + dw(ijkbeg:ijkend, 26, 20) &
                           + dw(ijkbeg:ijkend, 34, 20) &
                           + dw(ijkbeg:ijkend, 50, 20) &
                           + dw(ijkbeg:ijkend, 51, 20) &
                           + dw(ijkbeg:ijkend, 52, 20) &
                          + 0.1279300000000000D+00*dw(ijkbeg:ijkend, 57, 20) &
                          + 0.1031800000000000D+00*dw(ijkbeg:ijkend, 60, 20) &
                           + dw(ijkbeg:ijkend, 61, 20) &
                          + 0.5120800000000000D+00*dw(ijkbeg:ijkend, 64, 20) &
                          + 0.2915000000000000D-01*dw(ijkbeg:ijkend, 66, 20) &
                          + 0.2810700000000000D+00*dw(ijkbeg:ijkend, 67, 20)
      JacC(ijkbeg:ijkend,  1, 21) =  - dw(ijkbeg:ijkend, 24, 21) &
                          + 0.1730700000000000D+00*dw(ijkbeg:ijkend, 99, 21)
      JacC(ijkbeg:ijkend, 20, 21) =  + dw(ijkbeg:ijkend, 24, 21) &
                           - dw(ijkbeg:ijkend, 25, 21) &
                           + dw(ijkbeg:ijkend, 35, 21) &
                          + 0.7000000000000000D+00*dw(ijkbeg:ijkend, 38, 21)
      JacC(ijkbeg:ijkend, 21, 21) =  - dw(ijkbeg:ijkend, 24, 21) &
                           - dw(ijkbeg:ijkend, 25, 21) &
                          - 0.2000000000000000D+01*dw(ijkbeg:ijkend, 27, 21) &
                          - 0.2000000000000000D+01*dw(ijkbeg:ijkend, 27, 21) &
                          - 0.2000000000000000D+01*dw(ijkbeg:ijkend, 28, 21) &
                          - 0.2000000000000000D+01*dw(ijkbeg:ijkend, 28, 21) &
                           - dw(ijkbeg:ijkend, 35, 21) &
                           - dw(ijkbeg:ijkend, 36, 21) &
                           - dw(ijkbeg:ijkend, 38, 21) &
                           - dw(ijkbeg:ijkend, 81, 21) &
                           - dw(ijkbeg:ijkend, 94, 21) &
                           - dw(ijkbeg:ijkend, 95, 21) &
                           - dw(ijkbeg:ijkend, 96, 21) &
                           - dw(ijkbeg:ijkend, 97, 21) &
                           - dw(ijkbeg:ijkend, 98, 21) &
                           - dw(ijkbeg:ijkend, 99, 21) &
                           - dw(ijkbeg:ijkend,100, 21) &
                           - dw(ijkbeg:ijkend,123, 21)
      JacC(ijkbeg:ijkend,  2, 20) =  - dw(ijkbeg:ijkend, 26, 20)
      JacC(ijkbeg:ijkend, 21,  2) =  + dw(ijkbeg:ijkend, 26,  2)
      JacC(ijkbeg:ijkend,  2, 21) =  + dw(ijkbeg:ijkend, 27, 21) &
                           + dw(ijkbeg:ijkend, 27, 21) &
                           + dw(ijkbeg:ijkend, 28, 21) &
                           + dw(ijkbeg:ijkend, 28, 21)
      JacC(ijkbeg:ijkend,  3, 18) =  - dw(ijkbeg:ijkend, 29, 18) &
                           + dw(ijkbeg:ijkend, 30, 18)
      JacC(ijkbeg:ijkend,  3,  3) =  - dw(ijkbeg:ijkend, 29,  3) &
                           - dw(ijkbeg:ijkend, 32,  3) &
                           - dw(ijkbeg:ijkend, 35,  3) &
                           - dw(ijkbeg:ijkend, 42,  3) &
                          - 0.2000000000000000D+01*dw(ijkbeg:ijkend, 44,  3) &
                          - 0.2000000000000000D+01*dw(ijkbeg:ijkend, 44,  3) &
                           - dw(ijkbeg:ijkend, 45,  3) &
                           - dw(ijkbeg:ijkend, 87,  3) &
                           - dw(ijkbeg:ijkend, 88,  3) &
                           - dw(ijkbeg:ijkend, 89,  3) &
                           - dw(ijkbeg:ijkend, 90,  3) &
                           - dw(ijkbeg:ijkend, 91,  3) &
                           - dw(ijkbeg:ijkend, 92,  3) &
                           - dw(ijkbeg:ijkend, 93,  3) &
                           - dw(ijkbeg:ijkend,127,  3)
      JacC(ijkbeg:ijkend,  4, 18) =  + dw(ijkbeg:ijkend, 29, 18) &
                           - dw(ijkbeg:ijkend, 30, 18) &
                           - dw(ijkbeg:ijkend, 31, 18)
      JacC(ijkbeg:ijkend,  4,  3) =  + dw(ijkbeg:ijkend, 29,  3) &
                           + dw(ijkbeg:ijkend, 35,  3) &
                           + dw(ijkbeg:ijkend, 42,  3) &
                          + 0.2000000000000000D+01*dw(ijkbeg:ijkend, 44,  3) &
                          + 0.2000000000000000D+01*dw(ijkbeg:ijkend, 44,  3) &
                          + 0.2000000000000000D+01*dw(ijkbeg:ijkend, 45,  3) &
                           + dw(ijkbeg:ijkend, 87,  3) &
                          + 0.9154099999999999D+00*dw(ijkbeg:ijkend, 88,  3) &
                           + dw(ijkbeg:ijkend, 89,  3) &
                          + 0.8470000000000000D+00*dw(ijkbeg:ijkend, 90,  3) &
                          + 0.9511500000000001D+00*dw(ijkbeg:ijkend, 91,  3) &
                           + dw(ijkbeg:ijkend, 92,  3) &
                          + 0.1815990000000000D+01*dw(ijkbeg:ijkend, 93,  3) &
                           + dw(ijkbeg:ijkend,127,  3)
      JacC(ijkbeg:ijkend, 18,  3) =  - dw(ijkbeg:ijkend, 29,  3)
      JacC(ijkbeg:ijkend,  5, 18) =  + dw(ijkbeg:ijkend, 31, 18)
      JacC(ijkbeg:ijkend,  5,  4) =  + dw(ijkbeg:ijkend, 31,  4) &
                           + dw(ijkbeg:ijkend, 43,  4) &
                           - dw(ijkbeg:ijkend, 46,  4) &
                           - dw(ijkbeg:ijkend, 47,  4)
      JacC(ijkbeg:ijkend,  3, 20) =  - dw(ijkbeg:ijkend, 32, 20)
      JacC(ijkbeg:ijkend,  7, 20) =  + dw(ijkbeg:ijkend, 32, 20) &
                           - dw(ijkbeg:ijkend, 39, 20)
      JacC(ijkbeg:ijkend,  7,  3) =  + dw(ijkbeg:ijkend, 32,  3)
      JacC(ijkbeg:ijkend, 20,  3) =  - dw(ijkbeg:ijkend, 32,  3) &
                           + dw(ijkbeg:ijkend, 35,  3)
      JacC(ijkbeg:ijkend,  4, 20) =  - dw(ijkbeg:ijkend, 33, 20) &
                           + dw(ijkbeg:ijkend, 34, 20) &
                           + dw(ijkbeg:ijkend, 39, 20) &
                           + dw(ijkbeg:ijkend, 41, 20) &
                           + dw(ijkbeg:ijkend, 68, 20)
      JacC(ijkbeg:ijkend,  8, 20) =  + dw(ijkbeg:ijkend, 33, 20) &
                           - dw(ijkbeg:ijkend, 40, 20)
      JacC(ijkbeg:ijkend,  8,  4) =  + dw(ijkbeg:ijkend, 33,  4)
      JacC(ijkbeg:ijkend, 20,  4) =  - dw(ijkbeg:ijkend, 33,  4)
      JacC(ijkbeg:ijkend,  5, 20) =  - dw(ijkbeg:ijkend, 34, 20) &
                           + dw(ijkbeg:ijkend, 40, 20) &
                          + 0.7189300000000000D+00*dw(ijkbeg:ijkend, 67, 20)
      JacC(ijkbeg:ijkend, 20,  5) =  - dw(ijkbeg:ijkend, 34,  5) &
                          + 0.7000000000000000D+00*dw(ijkbeg:ijkend, 38,  5)
      JacC(ijkbeg:ijkend, 21,  5) =  + dw(ijkbeg:ijkend, 34,  5) &
                           - dw(ijkbeg:ijkend, 38,  5) &
                           + dw(ijkbeg:ijkend, 69,  5) &
                          + 0.6321700000000000D+00*dw(ijkbeg:ijkend, 71,  5) &
                           + dw(ijkbeg:ijkend,116,  5) &
                          + 0.8129000000000000D+00*dw(ijkbeg:ijkend,117,  5) &
                           + dw(ijkbeg:ijkend,118,  5) &
                           + dw(ijkbeg:ijkend,119,  5) &
                           + dw(ijkbeg:ijkend,120,  5) &
                          + 0.4915000000000000D-01*dw(ijkbeg:ijkend,121,  5) &
                          + 0.2592800000000000D+00*dw(ijkbeg:ijkend,122,  5)
      JacC(ijkbeg:ijkend,  3, 21) =  - dw(ijkbeg:ijkend, 35, 21)
      JacC(ijkbeg:ijkend,  4, 21) =  + dw(ijkbeg:ijkend, 35, 21) &
                           - dw(ijkbeg:ijkend, 36, 21) &
                          + 0.7000000000000000D+00*dw(ijkbeg:ijkend, 38, 21)
      JacC(ijkbeg:ijkend, 21,  3) =  - dw(ijkbeg:ijkend, 35,  3) &
                           + dw(ijkbeg:ijkend, 87,  3) &
                          + 0.7426500000000000D+00*dw(ijkbeg:ijkend, 88,  3) &
                           + dw(ijkbeg:ijkend, 89,  3) &
                          + 0.8470000000000000D+00*dw(ijkbeg:ijkend, 90,  3) &
                          + 0.9511500000000001D+00*dw(ijkbeg:ijkend, 91,  3) &
                          + 0.1233400000000000D+00*dw(ijkbeg:ijkend, 92,  3) &
                          + 0.1840100000000000D+00*dw(ijkbeg:ijkend, 93,  3)
      JacC(ijkbeg:ijkend,  9, 21) =  + dw(ijkbeg:ijkend, 36, 21)
      JacC(ijkbeg:ijkend,  9,  4) =  + dw(ijkbeg:ijkend, 36,  4)
      JacC(ijkbeg:ijkend, 21,  4) =  - dw(ijkbeg:ijkend, 36,  4)
      JacC(ijkbeg:ijkend,  5, 21) =  - dw(ijkbeg:ijkend, 38, 21)
      JacC(ijkbeg:ijkend,  8, 21) = + 0.3000000000000000D+00*dw(ijkbeg:ijkend, 38, 21)
      JacC(ijkbeg:ijkend,  8,  5) = + 0.3000000000000000D+00*dw(ijkbeg:ijkend, 38,  5) &
                           + dw(ijkbeg:ijkend, 69,  5) &
                           + dw(ijkbeg:ijkend, 70,  5) &
                          + 0.9156700000000000D+00*dw(ijkbeg:ijkend, 71,  5) &
                           + dw(ijkbeg:ijkend, 72,  5)
      JacC(ijkbeg:ijkend,  4,  7) =  + dw(ijkbeg:ijkend, 39,  7)
      JacC(ijkbeg:ijkend,  5,  8) =  + dw(ijkbeg:ijkend, 40,  8)
      JacC(ijkbeg:ijkend,  9, 20) =  - dw(ijkbeg:ijkend, 41, 20)
      JacC(ijkbeg:ijkend,  1,  3) =  - dw(ijkbeg:ijkend, 42,  3)
      JacC(ijkbeg:ijkend,  3,  1) =  - dw(ijkbeg:ijkend, 42,  1)
      JacC(ijkbeg:ijkend,  4,  1) =  + dw(ijkbeg:ijkend, 42,  1) &
                           - dw(ijkbeg:ijkend, 43,  1) &
                          + 0.7000000000000000D+00*dw(ijkbeg:ijkend, 79,  1)
      JacC(ijkbeg:ijkend,  1,  4) =  - dw(ijkbeg:ijkend, 43,  4)
      JacC(ijkbeg:ijkend,  5,  1) =  + dw(ijkbeg:ijkend, 43,  1)
      JacC(ijkbeg:ijkend,  5,  3) =  - dw(ijkbeg:ijkend, 45,  3)
      JacC(ijkbeg:ijkend,  6,  5) =  + dw(ijkbeg:ijkend, 47,  5)
      JacC(ijkbeg:ijkend,  6,  4) =  + dw(ijkbeg:ijkend, 47,  4)
      JacC(ijkbeg:ijkend,  4,  6) =  + dw(ijkbeg:ijkend, 48,  6)
      JacC(ijkbeg:ijkend,  5,  6) =  + dw(ijkbeg:ijkend, 48,  6)
      JacC(ijkbeg:ijkend,  6,  6) =  - dw(ijkbeg:ijkend, 48,  6)
      JacC(ijkbeg:ijkend, 10, 20) =  - dw(ijkbeg:ijkend, 51, 20)
      JacC(ijkbeg:ijkend, 10, 10) =  - dw(ijkbeg:ijkend, 51, 10)
      JacC(ijkbeg:ijkend, 11, 20) =  + dw(ijkbeg:ijkend, 51, 20)
      JacC(ijkbeg:ijkend, 11, 10) =  + dw(ijkbeg:ijkend, 51, 10)
      JacC(ijkbeg:ijkend, 20, 10) =  - dw(ijkbeg:ijkend, 51, 10)
      JacC(ijkbeg:ijkend, 21, 10) =  + dw(ijkbeg:ijkend, 51, 10)
      JacC(ijkbeg:ijkend, 12, 12) =  - dw(ijkbeg:ijkend, 52, 12)
      JacC(ijkbeg:ijkend, 12, 20) =  - dw(ijkbeg:ijkend, 52, 20) &
                          + 0.8780000000000000D-03*dw(ijkbeg:ijkend, 57, 20) &
                           + dw(ijkbeg:ijkend, 61, 20) &
                          + 0.1017320000000000D+01*dw(ijkbeg:ijkend, 64, 20)
      JacC(ijkbeg:ijkend, 20, 12) =  - dw(ijkbeg:ijkend, 52, 12)
      JacC(ijkbeg:ijkend, 21, 12) =  + dw(ijkbeg:ijkend, 52, 12)
      JacC(ijkbeg:ijkend, 12, 26) = + 0.1000000000000000D-01*dw(ijkbeg:ijkend, 53, 26) &
                          + 0.3600000000000000D+00*dw(ijkbeg:ijkend, 77, 26)
      JacC(ijkbeg:ijkend, 12, 18) = + 0.1000000000000000D-01*dw(ijkbeg:ijkend, 53, 18)
      JacC(ijkbeg:ijkend, 18, 26) =  - dw(ijkbeg:ijkend, 53, 26) &
                          + 0.9000000000000000D-01*dw(ijkbeg:ijkend, 77, 26)
      JacC(ijkbeg:ijkend, 20, 26) = + 0.2000000000000000D-01*dw(ijkbeg:ijkend, 53, 26) &
                           - dw(ijkbeg:ijkend, 59, 26) &
                          + 0.2800000000000000D+00*dw(ijkbeg:ijkend, 77, 26)
      JacC(ijkbeg:ijkend, 20, 18) = + 0.2000000000000000D-01*dw(ijkbeg:ijkend, 53, 18)
      JacC(ijkbeg:ijkend, 21, 26) = + 0.2800000000000000D+00*dw(ijkbeg:ijkend, 53, 26) &
                          + 0.3000000000000000D+00*dw(ijkbeg:ijkend, 77, 26)
      JacC(ijkbeg:ijkend, 21, 18) = + 0.2800000000000000D+00*dw(ijkbeg:ijkend, 53, 18)
      JacC(ijkbeg:ijkend, 25, 26) = + 0.9186800000000001D+00*dw(ijkbeg:ijkend, 53, 26) &
                          + 0.3738800000000000D+00*dw(ijkbeg:ijkend, 77, 26)
      JacC(ijkbeg:ijkend, 25, 18) = + 0.9186800000000001D+00*dw(ijkbeg:ijkend, 53, 18)
      JacC(ijkbeg:ijkend, 26, 26) =  - dw(ijkbeg:ijkend, 53, 26) &
                           - dw(ijkbeg:ijkend, 59, 26) &
                           - dw(ijkbeg:ijkend, 74, 26) &
                           - dw(ijkbeg:ijkend, 77, 26)
      JacC(ijkbeg:ijkend, 26, 18) =  - dw(ijkbeg:ijkend, 53, 18)
      JacC(ijkbeg:ijkend, 28, 26) = + 0.5000000000000000D-01*dw(ijkbeg:ijkend, 53, 26) &
                          + 0.9000000000000000D+00*dw(ijkbeg:ijkend, 77, 26)
      JacC(ijkbeg:ijkend, 28, 18) = + 0.5000000000000000D-01*dw(ijkbeg:ijkend, 53, 18)
      JacC(ijkbeg:ijkend, 31, 26) = + 0.1325500000000000D+00*dw(ijkbeg:ijkend, 53, 26) &
                          + 0.9174099999999999D+00*dw(ijkbeg:ijkend, 74, 26) &
                          + 0.3975400000000000D+00*dw(ijkbeg:ijkend, 77, 26)
      JacC(ijkbeg:ijkend, 31, 18) = + 0.1325500000000000D+00*dw(ijkbeg:ijkend, 53, 18) &
                           - dw(ijkbeg:ijkend, 54, 18)
      JacC(ijkbeg:ijkend, 47, 26) = + 0.1500000000000000D+00*dw(ijkbeg:ijkend, 53, 26) &
                          + 0.1300000000000000D+00*dw(ijkbeg:ijkend, 77, 26)
      JacC(ijkbeg:ijkend, 47, 18) = + 0.1500000000000000D+00*dw(ijkbeg:ijkend, 53, 18)
      JacC(ijkbeg:ijkend, 18, 31) =  - dw(ijkbeg:ijkend, 54, 31)
      JacC(ijkbeg:ijkend, 29, 31) =  + dw(ijkbeg:ijkend, 54, 31) &
                          + 0.6253000000000000D-01*dw(ijkbeg:ijkend, 64, 31) &
                          + 0.5265000000000000D-01*dw(ijkbeg:ijkend, 71, 31) &
                          + 0.1569200000000000D+00*dw(ijkbeg:ijkend, 78, 31)
      JacC(ijkbeg:ijkend, 29, 18) =  + dw(ijkbeg:ijkend, 54, 18)
      JacC(ijkbeg:ijkend, 20, 22) =  - dw(ijkbeg:ijkend, 55, 22)
      JacC(ijkbeg:ijkend, 22, 22) =  - dw(ijkbeg:ijkend, 55, 22)
      JacC(ijkbeg:ijkend, 22, 20) =  - dw(ijkbeg:ijkend, 55, 20)
      JacC(ijkbeg:ijkend, 38, 22) =  + dw(ijkbeg:ijkend, 55, 22)
      JacC(ijkbeg:ijkend, 38, 20) =  + dw(ijkbeg:ijkend, 55, 20) &
                          + 0.6500000000000000D+00*dw(ijkbeg:ijkend, 65, 20)
      JacC(ijkbeg:ijkend, 20, 23) =  - dw(ijkbeg:ijkend, 56, 23)
      JacC(ijkbeg:ijkend, 23, 23) =  - dw(ijkbeg:ijkend, 56, 23)
      JacC(ijkbeg:ijkend, 23, 20) =  - dw(ijkbeg:ijkend, 56, 20)
      JacC(ijkbeg:ijkend, 39, 23) =  + dw(ijkbeg:ijkend, 56, 23)
      JacC(ijkbeg:ijkend, 39, 20) =  + dw(ijkbeg:ijkend, 56, 20) &
                          + 0.8781099999999999D+00*dw(ijkbeg:ijkend, 57, 20) &
                          + 0.4034100000000000D+00*dw(ijkbeg:ijkend, 66, 20) &
                           + dw(ijkbeg:ijkend, 68, 20)
      JacC(ijkbeg:ijkend, 12, 24) = + 0.8780000000000000D-03*dw(ijkbeg:ijkend, 57, 24)
      JacC(ijkbeg:ijkend, 20, 24) = - 0.9991220000000000D+00*dw(ijkbeg:ijkend, 57, 24)
      JacC(ijkbeg:ijkend, 21, 24) = + 0.1279300000000000D+00*dw(ijkbeg:ijkend, 57, 24)
      JacC(ijkbeg:ijkend, 24, 24) =  - dw(ijkbeg:ijkend, 57, 24)
      JacC(ijkbeg:ijkend, 24, 20) =  - dw(ijkbeg:ijkend, 57, 20)
      JacC(ijkbeg:ijkend, 28, 24) = + 0.1400000000000000D-02*dw(ijkbeg:ijkend, 57, 24)
      JacC(ijkbeg:ijkend, 28, 20) = + 0.1400000000000000D-02*dw(ijkbeg:ijkend, 57, 20) &
                           - dw(ijkbeg:ijkend, 61, 20) &
                          + 0.3500000000000000D+00*dw(ijkbeg:ijkend, 65, 20) &
                          + 0.2915000000000000D-01*dw(ijkbeg:ijkend, 66, 20) &
                          + 0.5783900000000000D+00*dw(ijkbeg:ijkend, 67, 20)
      JacC(ijkbeg:ijkend, 29, 24) = + 0.8173000000000000D-01*dw(ijkbeg:ijkend, 57, 24)
      JacC(ijkbeg:ijkend, 29, 20) = + 0.8173000000000000D-01*dw(ijkbeg:ijkend, 57, 20) &
                           - dw(ijkbeg:ijkend, 62, 20) &
                          + 0.6253000000000000D-01*dw(ijkbeg:ijkend, 64, 20) &
                          + 0.7335000000000000D-01*dw(ijkbeg:ijkend, 66, 20)
      JacC(ijkbeg:ijkend, 30, 24) = + 0.3498000000000000D-01*dw(ijkbeg:ijkend, 57, 24)
      JacC(ijkbeg:ijkend, 30, 20) = + 0.3498000000000000D-01*dw(ijkbeg:ijkend, 57, 20) &
                           - dw(ijkbeg:ijkend, 63, 20) &
                          + 0.8529999999999999D-02*dw(ijkbeg:ijkend, 64, 20) &
                          + 0.3759100000000000D+00*dw(ijkbeg:ijkend, 66, 20)
      JacC(ijkbeg:ijkend, 31, 24) = + 0.8350000000000000D-02*dw(ijkbeg:ijkend, 57, 24)
      JacC(ijkbeg:ijkend, 31, 20) = + 0.8350000000000000D-02*dw(ijkbeg:ijkend, 57, 20) &
                          - 0.8308100000000000D+00*dw(ijkbeg:ijkend, 64, 20) &
                          + 0.2186300000000000D+00*dw(ijkbeg:ijkend, 67, 20)
      JacC(ijkbeg:ijkend, 36, 24) = + 0.8780000000000000D-02*dw(ijkbeg:ijkend, 57, 24)
      JacC(ijkbeg:ijkend, 36, 20) = + 0.8780000000000000D-02*dw(ijkbeg:ijkend, 57, 20)
      JacC(ijkbeg:ijkend, 39, 24) = + 0.8781099999999999D+00*dw(ijkbeg:ijkend, 57, 24)
      JacC(ijkbeg:ijkend, 20, 25) =  - dw(ijkbeg:ijkend, 58, 25) &
                          + 0.3943500000000000D+00*dw(ijkbeg:ijkend, 76, 25)
      JacC(ijkbeg:ijkend, 25, 25) =  - dw(ijkbeg:ijkend, 58, 25) &
                           - dw(ijkbeg:ijkend, 73, 25) &
                           - dw(ijkbeg:ijkend, 76, 25)
      JacC(ijkbeg:ijkend, 25, 20) =  - dw(ijkbeg:ijkend, 58, 20)
      JacC(ijkbeg:ijkend, 40, 25) = + 0.1025290000000000D+01*dw(ijkbeg:ijkend, 58, 25)
      JacC(ijkbeg:ijkend, 40, 20) = + 0.1025290000000000D+01*dw(ijkbeg:ijkend, 58, 20)
      JacC(ijkbeg:ijkend, 26, 20) =  - dw(ijkbeg:ijkend, 59, 20)
      JacC(ijkbeg:ijkend, 41, 26) =  + dw(ijkbeg:ijkend, 59, 26)
      JacC(ijkbeg:ijkend, 41, 20) =  + dw(ijkbeg:ijkend, 59, 20)
      JacC(ijkbeg:ijkend, 20, 27) =  - dw(ijkbeg:ijkend, 60, 27)
      JacC(ijkbeg:ijkend, 21, 27) = + 0.1031800000000000D+00*dw(ijkbeg:ijkend, 60, 27)
      JacC(ijkbeg:ijkend, 27, 27) =  - dw(ijkbeg:ijkend, 60, 27) &
                           - dw(ijkbeg:ijkend, 72, 27)
      JacC(ijkbeg:ijkend, 27, 20) =  - dw(ijkbeg:ijkend, 60, 20)
      JacC(ijkbeg:ijkend, 42, 27) = + 0.2760000000000000D-02*dw(ijkbeg:ijkend, 60, 27) &
                           + dw(ijkbeg:ijkend, 72, 27)
      JacC(ijkbeg:ijkend, 42, 20) = + 0.2760000000000000D-02*dw(ijkbeg:ijkend, 60, 20)
      JacC(ijkbeg:ijkend, 43, 27) = + 0.9396800000000000D+00*dw(ijkbeg:ijkend, 60, 27)
      JacC(ijkbeg:ijkend, 43, 20) = + 0.9396800000000000D+00*dw(ijkbeg:ijkend, 60, 20)
      JacC(ijkbeg:ijkend, 47, 27) = + 0.1031800000000000D+00*dw(ijkbeg:ijkend, 60, 27)
      JacC(ijkbeg:ijkend, 47, 20) = + 0.1031800000000000D+00*dw(ijkbeg:ijkend, 60, 20) &
                          + 0.1016200000000000D+00*dw(ijkbeg:ijkend, 64, 20) &
                          + 0.9333000000000000D-01*dw(ijkbeg:ijkend, 66, 20) &
                           + dw(ijkbeg:ijkend, 67, 20)
      JacC(ijkbeg:ijkend, 20, 28) =  - dw(ijkbeg:ijkend, 61, 28)
      JacC(ijkbeg:ijkend, 20, 29) =  - dw(ijkbeg:ijkend, 62, 29)
      JacC(ijkbeg:ijkend, 45, 29) =  + dw(ijkbeg:ijkend, 62, 29) &
                           + dw(ijkbeg:ijkend, 70, 29)
      JacC(ijkbeg:ijkend, 45, 20) =  + dw(ijkbeg:ijkend, 62, 20) &
                           + dw(ijkbeg:ijkend, 63, 20) &
                          + 0.5141900000000000D+00*dw(ijkbeg:ijkend, 64, 20) &
                          + 0.5413000000000000D-01*dw(ijkbeg:ijkend, 66, 20)
      JacC(ijkbeg:ijkend, 20, 30) =  - dw(ijkbeg:ijkend, 63, 30)
      JacC(ijkbeg:ijkend, 20, 31) =  - dw(ijkbeg:ijkend, 64, 31) &
                          + 0.2059500000000000D+00*dw(ijkbeg:ijkend, 78, 31)
      JacC(ijkbeg:ijkend, 30, 31) = + 0.8529999999999999D-02*dw(ijkbeg:ijkend, 64, 31) &
                          + 0.6320000000000000D-02*dw(ijkbeg:ijkend, 71, 31)
      JacC(ijkbeg:ijkend, 47, 31) = + 0.1016200000000000D+00*dw(ijkbeg:ijkend, 64, 31) &
                          + 0.1053000000000000D+00*dw(ijkbeg:ijkend, 71, 31)
      JacC(ijkbeg:ijkend, 34, 20) =  - dw(ijkbeg:ijkend, 65, 20)
      JacC(ijkbeg:ijkend, 38, 34) = + 0.6500000000000000D+00*dw(ijkbeg:ijkend, 65, 34)
      JacC(ijkbeg:ijkend, 28, 35) = + 0.2915000000000000D-01*dw(ijkbeg:ijkend, 66, 35)
      JacC(ijkbeg:ijkend, 30, 35) = + 0.3759100000000000D+00*dw(ijkbeg:ijkend, 66, 35)
      JacC(ijkbeg:ijkend, 35, 20) =  - dw(ijkbeg:ijkend, 66, 20)
      JacC(ijkbeg:ijkend, 39, 35) = + 0.4034100000000000D+00*dw(ijkbeg:ijkend, 66, 35)
      JacC(ijkbeg:ijkend, 45, 35) = + 0.5413000000000000D-01*dw(ijkbeg:ijkend, 66, 35)
      JacC(ijkbeg:ijkend, 47, 35) = + 0.9333000000000000D-01*dw(ijkbeg:ijkend, 66, 35)
      JacC(ijkbeg:ijkend,  5, 33) = + 0.7189300000000000D+00*dw(ijkbeg:ijkend, 67, 33) &
                          - 0.4000000000000000D+00*dw(ijkbeg:ijkend, 75, 33)
      JacC(ijkbeg:ijkend, 20, 33) =  - dw(ijkbeg:ijkend, 67, 33) &
                          + 0.3600000000000000D-01*dw(ijkbeg:ijkend, 79, 33)
      JacC(ijkbeg:ijkend, 21, 33) = + 0.2810700000000000D+00*dw(ijkbeg:ijkend, 67, 33) &
                          + 0.8000000000000000D-01*dw(ijkbeg:ijkend, 79, 33)
      JacC(ijkbeg:ijkend, 28, 33) = + 0.5783900000000000D+00*dw(ijkbeg:ijkend, 67, 33) &
                          + 0.4000000000000000D+00*dw(ijkbeg:ijkend, 75, 33) &
                          + 0.7000000000000000D+00*dw(ijkbeg:ijkend, 79, 33)
      JacC(ijkbeg:ijkend, 31, 33) = + 0.2186300000000000D+00*dw(ijkbeg:ijkend, 67, 33)
      JacC(ijkbeg:ijkend, 33, 33) = - 0.7189300000000001D+00*dw(ijkbeg:ijkend, 67, 33) &
                          - 0.6000000000000000D+00*dw(ijkbeg:ijkend, 75, 33) &
                          - 0.7000000000000000D+00*dw(ijkbeg:ijkend, 79, 33) &
                           - dw(ijkbeg:ijkend, 86, 33)
      JacC(ijkbeg:ijkend, 33, 20) = - 0.7189300000000001D+00*dw(ijkbeg:ijkend, 67, 20)
      JacC(ijkbeg:ijkend, 47, 33) =  + dw(ijkbeg:ijkend, 67, 33) &
                           + dw(ijkbeg:ijkend, 75, 33)
      JacC(ijkbeg:ijkend, 20, 32) =  - dw(ijkbeg:ijkend, 68, 32)
      JacC(ijkbeg:ijkend, 32, 20) =  - dw(ijkbeg:ijkend, 68, 20)
      JacC(ijkbeg:ijkend, 39, 32) =  + dw(ijkbeg:ijkend, 68, 32)
      JacC(ijkbeg:ijkend,  5, 28) =  - dw(ijkbeg:ijkend, 69, 28)
      JacC(ijkbeg:ijkend,  8, 28) =  + dw(ijkbeg:ijkend, 69, 28)
      JacC(ijkbeg:ijkend, 12,  5) =  + dw(ijkbeg:ijkend, 69,  5) &
                          + 0.1337230000000000D+01*dw(ijkbeg:ijkend, 71,  5)
      JacC(ijkbeg:ijkend, 28,  5) =  - dw(ijkbeg:ijkend, 69,  5) &
                          + 0.4000000000000000D+00*dw(ijkbeg:ijkend, 75,  5) &
                           + dw(ijkbeg:ijkend,116,  5) &
                          + 0.3142000000000000D-01*dw(ijkbeg:ijkend,117,  5) &
                          + 0.1409090000000000D+01*dw(ijkbeg:ijkend,118,  5) &
                          + 0.6860000000000001D+00*dw(ijkbeg:ijkend,119,  5) &
                          + 0.3175000000000000D-01*dw(ijkbeg:ijkend,121,  5) &
                          + 0.2074000000000000D+00*dw(ijkbeg:ijkend,122,  5)
      JacC(ijkbeg:ijkend,  5, 29) =  - dw(ijkbeg:ijkend, 70, 29)
      JacC(ijkbeg:ijkend,  8, 29) =  + dw(ijkbeg:ijkend, 70, 29)
      JacC(ijkbeg:ijkend, 29,  5) =  - dw(ijkbeg:ijkend, 70,  5) &
                          + 0.5265000000000000D-01*dw(ijkbeg:ijkend, 71,  5) &
                          + 0.3374300000000000D+00*dw(ijkbeg:ijkend,117,  5) &
                          + 0.4303900000000000D+00*dw(ijkbeg:ijkend,118,  5) &
                          + 0.2936000000000000D-01*dw(ijkbeg:ijkend,121,  5) &
                          + 0.9185000000000000D+00*dw(ijkbeg:ijkend,122,  5)
      JacC(ijkbeg:ijkend, 45,  5) =  + dw(ijkbeg:ijkend, 70,  5) &
                          + 0.3888100000000000D+00*dw(ijkbeg:ijkend, 71,  5) &
                          - 0.9682500000000001D+00*dw(ijkbeg:ijkend,121,  5)
      JacC(ijkbeg:ijkend,  4, 31) = + 0.1053000000000000D+00*dw(ijkbeg:ijkend, 71, 31)
      JacC(ijkbeg:ijkend,  5, 31) =  - dw(ijkbeg:ijkend, 71, 31)
      JacC(ijkbeg:ijkend,  8, 31) = + 0.9156700000000000D+00*dw(ijkbeg:ijkend, 71, 31)
      JacC(ijkbeg:ijkend, 30,  5) = + 0.6320000000000000D-02*dw(ijkbeg:ijkend, 71,  5) &
                          + 0.6297800000000000D+00*dw(ijkbeg:ijkend,117,  5) &
                          + 0.2051000000000000D-01*dw(ijkbeg:ijkend,118,  5) &
                          + 0.3474000000000000D+00*dw(ijkbeg:ijkend,122,  5)
      JacC(ijkbeg:ijkend, 31,  5) = - 0.8947000000000001D+00*dw(ijkbeg:ijkend, 71,  5) &
                          + 0.9174099999999999D+00*dw(ijkbeg:ijkend, 74,  5) &
                          + 0.3531000000000000D-01*dw(ijkbeg:ijkend,117,  5) &
                          + 0.6116000000000000D+00*dw(ijkbeg:ijkend,119,  5) &
                          + 0.2819040000000000D+01*dw(ijkbeg:ijkend,120,  5) &
                          + 0.3455000000000000D-01*dw(ijkbeg:ijkend,121,  5)
      JacC(ijkbeg:ijkend, 47,  5) = + 0.1053000000000000D+00*dw(ijkbeg:ijkend, 71,  5) &
                           + dw(ijkbeg:ijkend, 75,  5) &
                          + 0.1627100000000000D+00*dw(ijkbeg:ijkend,117,  5) &
                          + 0.1021000000000000D-01*dw(ijkbeg:ijkend,121,  5) &
                           - dw(ijkbeg:ijkend,128,  5)
      JacC(ijkbeg:ijkend,  5, 27) =  - dw(ijkbeg:ijkend, 72, 27)
      JacC(ijkbeg:ijkend,  8, 27) =  + dw(ijkbeg:ijkend, 72, 27)
      JacC(ijkbeg:ijkend, 27,  5) =  - dw(ijkbeg:ijkend, 72,  5)
      JacC(ijkbeg:ijkend, 42,  5) =  + dw(ijkbeg:ijkend, 72,  5)
      JacC(ijkbeg:ijkend,  5, 25) =  - dw(ijkbeg:ijkend, 73, 25)
      JacC(ijkbeg:ijkend, 25,  5) =  - dw(ijkbeg:ijkend, 73,  5) &
                          + 0.4272900000000000D+00*dw(ijkbeg:ijkend,119,  5)
      JacC(ijkbeg:ijkend, 46, 25) = + 0.9376800000000000D+00*dw(ijkbeg:ijkend, 73, 25)
      JacC(ijkbeg:ijkend, 46,  5) = + 0.9376800000000000D+00*dw(ijkbeg:ijkend, 73,  5) &
                           + dw(ijkbeg:ijkend, 74,  5) &
                           - dw(ijkbeg:ijkend,122,  5)
      JacC(ijkbeg:ijkend,  5, 26) =  - dw(ijkbeg:ijkend, 74, 26)
      JacC(ijkbeg:ijkend, 26,  5) =  - dw(ijkbeg:ijkend, 74,  5)
      JacC(ijkbeg:ijkend, 46, 26) =  + dw(ijkbeg:ijkend, 74, 26)
      JacC(ijkbeg:ijkend,  4, 33) = + 0.4000000000000000D+00*dw(ijkbeg:ijkend, 75, 33) &
                          + 0.7000000000000000D+00*dw(ijkbeg:ijkend, 79, 33) &
                           + dw(ijkbeg:ijkend, 86, 33)
      JacC(ijkbeg:ijkend, 32, 33) = + 0.6000000000000000D+00*dw(ijkbeg:ijkend, 75, 33)
      JacC(ijkbeg:ijkend, 32,  5) = + 0.6000000000000000D+00*dw(ijkbeg:ijkend, 75,  5) &
                          + 0.2592800000000000D+00*dw(ijkbeg:ijkend,122,  5)
      JacC(ijkbeg:ijkend, 33,  5) = - 0.6000000000000000D+00*dw(ijkbeg:ijkend, 75,  5)
      JacC(ijkbeg:ijkend,  1, 25) =  - dw(ijkbeg:ijkend, 76, 25)
      JacC(ijkbeg:ijkend,  2, 25) = + 0.1833000000000000D-01*dw(ijkbeg:ijkend, 76, 25)
      JacC(ijkbeg:ijkend,  2,  1) = + 0.1833000000000000D-01*dw(ijkbeg:ijkend, 76,  1) &
                          + 0.1000000000000000D-02*dw(ijkbeg:ijkend, 77,  1)
      JacC(ijkbeg:ijkend, 12, 25) = + 0.3512000000000000D+00*dw(ijkbeg:ijkend, 76, 25)
      JacC(ijkbeg:ijkend, 12,  1) = + 0.3512000000000000D+00*dw(ijkbeg:ijkend, 76,  1) &
                          + 0.3600000000000000D+00*dw(ijkbeg:ijkend, 77,  1) &
                          + 0.6472800000000000D+00*dw(ijkbeg:ijkend, 78,  1) &
                          + 0.1300000000000000D+00*dw(ijkbeg:ijkend, 79,  1)
      JacC(ijkbeg:ijkend, 17, 25) = + 0.5409000000000000D-01*dw(ijkbeg:ijkend, 76, 25)
      JacC(ijkbeg:ijkend, 17,  1) = + 0.5409000000000000D-01*dw(ijkbeg:ijkend, 76,  1) &
                          + 0.5000000000000000D-01*dw(ijkbeg:ijkend, 77,  1) &
                          + 0.4000000000000000D-01*dw(ijkbeg:ijkend, 79,  1)
      JacC(ijkbeg:ijkend, 21, 25) = + 0.2345100000000000D+00*dw(ijkbeg:ijkend, 76, 25)
      JacC(ijkbeg:ijkend, 22, 25) = + 0.4300000000000000D-01*dw(ijkbeg:ijkend, 76, 25)
      JacC(ijkbeg:ijkend, 22,  1) = + 0.4300000000000000D-01*dw(ijkbeg:ijkend, 76,  1)
      JacC(ijkbeg:ijkend, 23, 25) = + 0.3196000000000000D-01*dw(ijkbeg:ijkend, 76, 25)
      JacC(ijkbeg:ijkend, 23,  1) = + 0.3196000000000000D-01*dw(ijkbeg:ijkend, 76,  1)
      JacC(ijkbeg:ijkend, 25,  1) =  - dw(ijkbeg:ijkend, 76,  1) &
                          + 0.3738800000000000D+00*dw(ijkbeg:ijkend, 77,  1)
      JacC(ijkbeg:ijkend, 28, 25) = + 0.4829000000000000D+00*dw(ijkbeg:ijkend, 76, 25)
      JacC(ijkbeg:ijkend, 28,  1) = + 0.4829000000000000D+00*dw(ijkbeg:ijkend, 76,  1) &
                          + 0.9000000000000000D+00*dw(ijkbeg:ijkend, 77,  1) &
                          + 0.7000000000000000D+00*dw(ijkbeg:ijkend, 79,  1)
      JacC(ijkbeg:ijkend, 29, 25) = + 0.5146800000000000D+00*dw(ijkbeg:ijkend, 76, 25)
      JacC(ijkbeg:ijkend, 29,  1) = + 0.5146800000000000D+00*dw(ijkbeg:ijkend, 76,  1) &
                          + 0.1569200000000000D+00*dw(ijkbeg:ijkend, 78,  1)
      JacC(ijkbeg:ijkend, 30, 25) = + 0.7377000000000000D-01*dw(ijkbeg:ijkend, 76, 25)
      JacC(ijkbeg:ijkend, 30,  1) = + 0.7377000000000000D-01*dw(ijkbeg:ijkend, 76,  1)
      JacC(ijkbeg:ijkend, 36, 25) = + 0.1534300000000000D+00*dw(ijkbeg:ijkend, 76, 25)
      JacC(ijkbeg:ijkend, 36,  1) = + 0.1534300000000000D+00*dw(ijkbeg:ijkend, 76,  1) &
                          + 0.1500000000000000D+00*dw(ijkbeg:ijkend, 77,  1) &
                          + 0.1078800000000000D+00*dw(ijkbeg:ijkend, 78,  1) &
                          + 0.1100000000000000D+00*dw(ijkbeg:ijkend, 79,  1)
      JacC(ijkbeg:ijkend, 37, 25) = + 0.8143000000000000D-01*dw(ijkbeg:ijkend, 76, 25)
      JacC(ijkbeg:ijkend, 37,  1) = + 0.8143000000000000D-01*dw(ijkbeg:ijkend, 76,  1) &
                          + 0.2059500000000000D+00*dw(ijkbeg:ijkend, 78,  1)
      JacC(ijkbeg:ijkend, 38, 25) = + 0.1396600000000000D+00*dw(ijkbeg:ijkend, 76, 25)
      JacC(ijkbeg:ijkend, 38,  1) = + 0.1396600000000000D+00*dw(ijkbeg:ijkend, 76,  1) &
                          + 0.3000000000000000D-01*dw(ijkbeg:ijkend, 77,  1)
      JacC(ijkbeg:ijkend, 39, 25) = + 0.9815000000000000D-01*dw(ijkbeg:ijkend, 76, 25)
      JacC(ijkbeg:ijkend, 39,  1) = + 0.9815000000000000D-01*dw(ijkbeg:ijkend, 76,  1)
      JacC(ijkbeg:ijkend, 45, 25) = + 0.5705000000000000D-01*dw(ijkbeg:ijkend, 76, 25)
      JacC(ijkbeg:ijkend, 45,  1) = + 0.5705000000000000D-01*dw(ijkbeg:ijkend, 76,  1) &
                          + 0.1700000000000000D+00*dw(ijkbeg:ijkend, 77,  1) &
                          + 0.2746000000000000D+00*dw(ijkbeg:ijkend, 78,  1) &
                          + 0.7000000000000000D+00*dw(ijkbeg:ijkend, 79,  1)
      JacC(ijkbeg:ijkend,  1, 26) =  - dw(ijkbeg:ijkend, 77, 26)
      JacC(ijkbeg:ijkend,  2, 26) = + 0.1000000000000000D-02*dw(ijkbeg:ijkend, 77, 26)
      JacC(ijkbeg:ijkend, 17, 26) = + 0.5000000000000000D-01*dw(ijkbeg:ijkend, 77, 26)
      JacC(ijkbeg:ijkend, 26,  1) =  - dw(ijkbeg:ijkend, 77,  1)
      JacC(ijkbeg:ijkend, 31,  1) = + 0.3975400000000000D+00*dw(ijkbeg:ijkend, 77,  1) &
                          + 0.7583000000000006D-01*dw(ijkbeg:ijkend, 78,  1)
      JacC(ijkbeg:ijkend, 36, 26) = + 0.1500000000000000D+00*dw(ijkbeg:ijkend, 77, 26)
      JacC(ijkbeg:ijkend, 38, 26) = + 0.3000000000000000D-01*dw(ijkbeg:ijkend, 77, 26)
      JacC(ijkbeg:ijkend, 45, 26) = + 0.1700000000000000D+00*dw(ijkbeg:ijkend, 77, 26)
      JacC(ijkbeg:ijkend, 47,  1) = + 0.1300000000000000D+00*dw(ijkbeg:ijkend, 77,  1)
      JacC(ijkbeg:ijkend,  1, 31) =  - dw(ijkbeg:ijkend, 78, 31)
      JacC(ijkbeg:ijkend, 35, 31) = + 0.1014900000000000D+00*dw(ijkbeg:ijkend, 78, 31)
      JacC(ijkbeg:ijkend, 35,  1) = + 0.1014900000000000D+00*dw(ijkbeg:ijkend, 78,  1)
      JacC(ijkbeg:ijkend, 36, 31) = + 0.1078800000000000D+00*dw(ijkbeg:ijkend, 78, 31)
      JacC(ijkbeg:ijkend, 37, 31) = + 0.2059500000000000D+00*dw(ijkbeg:ijkend, 78, 31)
      JacC(ijkbeg:ijkend,  1, 33) =  - dw(ijkbeg:ijkend, 79, 33)
      JacC(ijkbeg:ijkend, 12, 33) = + 0.1300000000000000D+00*dw(ijkbeg:ijkend, 79, 33)
      JacC(ijkbeg:ijkend, 17, 33) = + 0.4000000000000000D-01*dw(ijkbeg:ijkend, 79, 33)
      JacC(ijkbeg:ijkend, 33,  1) = - 0.7000000000000000D+00*dw(ijkbeg:ijkend, 79,  1)
      JacC(ijkbeg:ijkend, 36, 33) = + 0.1100000000000000D+00*dw(ijkbeg:ijkend, 79, 33)
      JacC(ijkbeg:ijkend, 45, 33) = + 0.7000000000000000D+00*dw(ijkbeg:ijkend, 79, 33) &
                           + dw(ijkbeg:ijkend, 86, 33)
      JacC(ijkbeg:ijkend,  4, 42) =  - dw(ijkbeg:ijkend, 80, 42)
      JacC(ijkbeg:ijkend, 27, 42) = + 0.1067000000000000D+00*dw(ijkbeg:ijkend, 80, 42) &
                          + 0.1066980000000000D+01*dw(ijkbeg:ijkend, 81, 42)
      JacC(ijkbeg:ijkend, 27,  4) = + 0.1067000000000000D+00*dw(ijkbeg:ijkend, 80,  4) &
                           + dw(ijkbeg:ijkend, 82,  4)
      JacC(ijkbeg:ijkend, 32, 42) =  + dw(ijkbeg:ijkend, 80, 42)
      JacC(ijkbeg:ijkend, 32,  4) =  + dw(ijkbeg:ijkend, 80,  4)
      JacC(ijkbeg:ijkend, 42, 42) =  - dw(ijkbeg:ijkend, 80, 42) &
                           - dw(ijkbeg:ijkend, 81, 42)
      JacC(ijkbeg:ijkend, 42,  4) =  - dw(ijkbeg:ijkend, 80,  4)
      JacC(ijkbeg:ijkend, 21, 42) =  - dw(ijkbeg:ijkend, 81, 42)
      JacC(ijkbeg:ijkend, 27, 21) = + 0.1066980000000000D+01*dw(ijkbeg:ijkend, 81, 21)
      JacC(ijkbeg:ijkend, 42, 21) =  - dw(ijkbeg:ijkend, 81, 21)
      JacC(ijkbeg:ijkend,  4, 43) =  - dw(ijkbeg:ijkend, 82, 43)
      JacC(ijkbeg:ijkend,  7, 43) =  + dw(ijkbeg:ijkend, 82, 43)
      JacC(ijkbeg:ijkend,  7,  4) =  + dw(ijkbeg:ijkend, 82,  4)
      JacC(ijkbeg:ijkend, 27, 43) =  + dw(ijkbeg:ijkend, 82, 43) &
                          + 0.2000000000000000D-01*dw(ijkbeg:ijkend, 83, 43) &
                           + dw(ijkbeg:ijkend, 84, 43)
      JacC(ijkbeg:ijkend, 43, 43) =  - dw(ijkbeg:ijkend, 82, 43) &
                           - dw(ijkbeg:ijkend, 83, 43) &
                           - dw(ijkbeg:ijkend, 84, 43)
      JacC(ijkbeg:ijkend, 43,  4) =  - dw(ijkbeg:ijkend, 82,  4)
      JacC(ijkbeg:ijkend, 21, 43) = + 0.2000000000000000D-01*dw(ijkbeg:ijkend, 83, 43)
      JacC(ijkbeg:ijkend, 44, 43) = + 0.9800000000000000D+00*dw(ijkbeg:ijkend, 83, 43)
      JacC(ijkbeg:ijkend,  1, 43) =  - dw(ijkbeg:ijkend, 84, 43)
      JacC(ijkbeg:ijkend, 20, 43) =  + dw(ijkbeg:ijkend, 84, 43)
      JacC(ijkbeg:ijkend, 27,  1) =  + dw(ijkbeg:ijkend, 84,  1)
      JacC(ijkbeg:ijkend, 43,  1) =  - dw(ijkbeg:ijkend, 84,  1)
      JacC(ijkbeg:ijkend,  4, 45) =  - dw(ijkbeg:ijkend, 85, 45) &
                           + dw(ijkbeg:ijkend, 92, 45) &
                           + dw(ijkbeg:ijkend,121, 45)
      JacC(ijkbeg:ijkend, 33, 45) =  + dw(ijkbeg:ijkend, 85, 45)
      JacC(ijkbeg:ijkend, 33,  4) =  + dw(ijkbeg:ijkend, 85,  4)
      JacC(ijkbeg:ijkend, 45, 45) =  - dw(ijkbeg:ijkend, 85, 45) &
                          - 0.9046800000000000D+00*dw(ijkbeg:ijkend, 92, 45) &
                           - dw(ijkbeg:ijkend, 99, 45) &
                          - 0.9404600000000000D+00*dw(ijkbeg:ijkend,106, 45) &
                           - dw(ijkbeg:ijkend,108, 45) &
                           - dw(ijkbeg:ijkend,109, 45) &
                           - dw(ijkbeg:ijkend,110, 45) &
                           - dw(ijkbeg:ijkend,111, 45) &
                          - 0.1941790000000000D+01*dw(ijkbeg:ijkend,112, 45) &
                          - 0.1941790000000000D+01*dw(ijkbeg:ijkend,112, 45) &
                           - dw(ijkbeg:ijkend,113, 45) &
                          - 0.9682500000000001D+00*dw(ijkbeg:ijkend,121, 45) &
                           - dw(ijkbeg:ijkend,125, 45)
      JacC(ijkbeg:ijkend, 45,  4) =  - dw(ijkbeg:ijkend, 85,  4)
      JacC(ijkbeg:ijkend,  3, 38) =  - dw(ijkbeg:ijkend, 87, 38)
      JacC(ijkbeg:ijkend,  4, 38) =  + dw(ijkbeg:ijkend, 87, 38) &
                          + 0.3244000000000000D+00*dw(ijkbeg:ijkend,107, 38) &
                           + dw(ijkbeg:ijkend,116, 38)
      JacC(ijkbeg:ijkend, 21, 38) =  + dw(ijkbeg:ijkend, 87, 38) &
                           - dw(ijkbeg:ijkend, 94, 38) &
                          + 0.6600000000000000D+00*dw(ijkbeg:ijkend,101, 38) &
                          + 0.6600000000000000D+00*dw(ijkbeg:ijkend,101, 38) &
                          + 0.9838300000000000D+00*dw(ijkbeg:ijkend,102, 38) &
                           + dw(ijkbeg:ijkend,103, 38) &
                           + dw(ijkbeg:ijkend,104, 38) &
                          + 0.1027670000000000D+01*dw(ijkbeg:ijkend,105, 38) &
                          + 0.8299800000000001D+00*dw(ijkbeg:ijkend,106, 38) &
                          + 0.6756000000000000D+00*dw(ijkbeg:ijkend,107, 38) &
                           + dw(ijkbeg:ijkend,116, 38) &
                           + dw(ijkbeg:ijkend,124, 38)
      JacC(ijkbeg:ijkend, 28, 38) =  + dw(ijkbeg:ijkend, 87, 38) &
                          + 0.1330000000000000D+01*dw(ijkbeg:ijkend,101, 38) &
                          + 0.1330000000000000D+01*dw(ijkbeg:ijkend,101, 38) &
                          + 0.8055600000000001D+00*dw(ijkbeg:ijkend,102, 38) &
                          + 0.1428940000000000D+01*dw(ijkbeg:ijkend,103, 38) &
                          + 0.1090000000000000D+01*dw(ijkbeg:ijkend,104, 38) &
                           + dw(ijkbeg:ijkend,105, 38) &
                          + 0.9572300000000000D+00*dw(ijkbeg:ijkend,106, 38) &
                          + 0.8862500000000000D+00*dw(ijkbeg:ijkend,107, 38) &
                           + dw(ijkbeg:ijkend,116, 38) &
                           + dw(ijkbeg:ijkend,124, 38)
      JacC(ijkbeg:ijkend, 28,  3) =  + dw(ijkbeg:ijkend, 87,  3) &
                          + 0.3002000000000000D-01*dw(ijkbeg:ijkend, 88,  3) &
                          + 0.1398700000000000D+01*dw(ijkbeg:ijkend, 89,  3) &
                          + 0.6060000000000000D+00*dw(ijkbeg:ijkend, 90,  3) &
                          + 0.5848000000000000D-01*dw(ijkbeg:ijkend, 92,  3) &
                          + 0.2341900000000000D+00*dw(ijkbeg:ijkend, 93,  3)
      JacC(ijkbeg:ijkend, 38, 38) =  - dw(ijkbeg:ijkend, 87, 38) &
                           - dw(ijkbeg:ijkend, 94, 38) &
                          - 0.2000000000000000D+01*dw(ijkbeg:ijkend,101, 38) &
                          - 0.2000000000000000D+01*dw(ijkbeg:ijkend,101, 38) &
                          - 0.9861000000000000D+00*dw(ijkbeg:ijkend,102, 38) &
                           - dw(ijkbeg:ijkend,103, 38) &
                           - dw(ijkbeg:ijkend,104, 38) &
                           - dw(ijkbeg:ijkend,105, 38) &
                          - 0.4396900000000000D+00*dw(ijkbeg:ijkend,106, 38) &
                           - dw(ijkbeg:ijkend,107, 38) &
                           - dw(ijkbeg:ijkend,116, 38) &
                           - dw(ijkbeg:ijkend,124, 38)
      JacC(ijkbeg:ijkend, 38,  3) =  - dw(ijkbeg:ijkend, 87,  3) &
                          + 0.9016000000000000D-01*dw(ijkbeg:ijkend, 88,  3) &
                          + 0.7813400000000000D+00*dw(ijkbeg:ijkend, 92,  3)
      JacC(ijkbeg:ijkend,  3, 39) =  - dw(ijkbeg:ijkend, 88, 39)
      JacC(ijkbeg:ijkend,  4, 39) = + 0.9154099999999999D+00*dw(ijkbeg:ijkend, 88, 39) &
                           + dw(ijkbeg:ijkend,117, 39)
      JacC(ijkbeg:ijkend, 21, 39) = + 0.7426500000000000D+00*dw(ijkbeg:ijkend, 88, 39) &
                           - dw(ijkbeg:ijkend, 95, 39) &
                          + 0.9838300000000000D+00*dw(ijkbeg:ijkend,102, 39) &
                          + 0.4807900000000000D+00*dw(ijkbeg:ijkend,108, 39) &
                          + 0.8129000000000000D+00*dw(ijkbeg:ijkend,117, 39)
      JacC(ijkbeg:ijkend, 28, 39) = + 0.3002000000000000D-01*dw(ijkbeg:ijkend, 88, 39) &
                          + 0.8055600000000001D+00*dw(ijkbeg:ijkend,102, 39) &
                          + 0.7600000000000000D-01*dw(ijkbeg:ijkend,108, 39) &
                          + 0.3142000000000000D-01*dw(ijkbeg:ijkend,117, 39)
      JacC(ijkbeg:ijkend, 29, 39) = + 0.3314400000000000D+00*dw(ijkbeg:ijkend, 88, 39) &
                          + 0.5607000000000000D+00*dw(ijkbeg:ijkend,102, 39) &
                          + 0.7146100000000000D+00*dw(ijkbeg:ijkend,108, 39) &
                          + 0.3374300000000000D+00*dw(ijkbeg:ijkend,117, 39)
      JacC(ijkbeg:ijkend, 29,  3) = + 0.3314400000000000D+00*dw(ijkbeg:ijkend, 88,  3) &
                          + 0.4212500000000000D+00*dw(ijkbeg:ijkend, 89,  3) &
                          + 0.7368000000000000D-01*dw(ijkbeg:ijkend, 92,  3) &
                          + 0.1011820000000000D+01*dw(ijkbeg:ijkend, 93,  3)
      JacC(ijkbeg:ijkend, 30, 39) = + 0.5453100000000000D+00*dw(ijkbeg:ijkend, 88, 39) &
                          + 0.9673000000000000D-01*dw(ijkbeg:ijkend,102, 39) &
                          + 0.1881900000000000D+00*dw(ijkbeg:ijkend,108, 39) &
                          + 0.6297800000000000D+00*dw(ijkbeg:ijkend,117, 39)
      JacC(ijkbeg:ijkend, 30,  3) = + 0.5453100000000000D+00*dw(ijkbeg:ijkend, 88,  3) &
                          + 0.5220000000000000D-01*dw(ijkbeg:ijkend, 89,  3) &
                          + 0.3786200000000000D+00*dw(ijkbeg:ijkend, 93,  3)
      JacC(ijkbeg:ijkend, 31, 39) = + 0.3407000000000000D-01*dw(ijkbeg:ijkend, 88, 39) &
                          + 0.7976000000000000D-01*dw(ijkbeg:ijkend,102, 39) &
                          + 0.6954000000000000D-01*dw(ijkbeg:ijkend,108, 39) &
                          + 0.3531000000000000D-01*dw(ijkbeg:ijkend,117, 39)
      JacC(ijkbeg:ijkend, 31,  3) = + 0.3407000000000000D-01*dw(ijkbeg:ijkend, 88,  3) &
                          + 0.4546300000000000D+00*dw(ijkbeg:ijkend, 90,  3) &
                          + 0.2069930000000000D+01*dw(ijkbeg:ijkend, 91,  3) &
                          + 0.8670000000000000D-01*dw(ijkbeg:ijkend, 92,  3)
      JacC(ijkbeg:ijkend, 32, 39) = + 0.8459000000000000D-01*dw(ijkbeg:ijkend, 88, 39)
      JacC(ijkbeg:ijkend, 32,  3) = + 0.8459000000000000D-01*dw(ijkbeg:ijkend, 88,  3) &
                          + 0.1530000000000000D+00*dw(ijkbeg:ijkend, 90,  3) &
                          + 0.4885000000000000D-01*dw(ijkbeg:ijkend, 91,  3) &
                          + 0.1840100000000000D+00*dw(ijkbeg:ijkend, 93,  3)
      JacC(ijkbeg:ijkend, 38, 39) = + 0.9016000000000000D-01*dw(ijkbeg:ijkend, 88, 39) &
                          - 0.9861000000000000D+00*dw(ijkbeg:ijkend,102, 39) &
                          + 0.5148000000000000D+00*dw(ijkbeg:ijkend,108, 39) &
                          + 0.9730999999999999D-01*dw(ijkbeg:ijkend,117, 39)
      JacC(ijkbeg:ijkend, 39, 39) = - 0.9181300000000000D+00*dw(ijkbeg:ijkend, 88, 39) &
                           - dw(ijkbeg:ijkend, 95, 39) &
                          - 0.9961500000000000D+00*dw(ijkbeg:ijkend,102, 39) &
                          - 0.9917200000000000D+00*dw(ijkbeg:ijkend,108, 39) &
                          - 0.9100600000000000D+00*dw(ijkbeg:ijkend,117, 39)
      JacC(ijkbeg:ijkend, 39,  3) = - 0.9181300000000000D+00*dw(ijkbeg:ijkend, 88,  3)
      JacC(ijkbeg:ijkend, 47, 39) = + 0.1300700000000000D+00*dw(ijkbeg:ijkend, 88, 39) &
                          + 0.1337000000000000D+00*dw(ijkbeg:ijkend,102, 39) &
                          + 0.1130600000000000D+00*dw(ijkbeg:ijkend,108, 39) &
                          + 0.1627100000000000D+00*dw(ijkbeg:ijkend,117, 39)
      JacC(ijkbeg:ijkend, 47,  3) = + 0.1300700000000000D+00*dw(ijkbeg:ijkend, 88,  3) &
                          + 0.2563000000000000D-01*dw(ijkbeg:ijkend, 92,  3) &
                           - dw(ijkbeg:ijkend,127,  3)
      JacC(ijkbeg:ijkend,  3, 40) =  - dw(ijkbeg:ijkend, 89, 40)
      JacC(ijkbeg:ijkend,  4, 40) =  + dw(ijkbeg:ijkend, 89, 40) &
                           + dw(ijkbeg:ijkend,118, 40)
      JacC(ijkbeg:ijkend, 21, 40) =  + dw(ijkbeg:ijkend, 89, 40) &
                           - dw(ijkbeg:ijkend, 96, 40) &
                           + dw(ijkbeg:ijkend,103, 40) &
                          + 0.5007800000000000D+00*dw(ijkbeg:ijkend,109, 40) &
                           + dw(ijkbeg:ijkend,118, 40)
      JacC(ijkbeg:ijkend, 28, 40) = + 0.1398700000000000D+01*dw(ijkbeg:ijkend, 89, 40) &
                          + 0.1428940000000000D+01*dw(ijkbeg:ijkend,103, 40) &
                          + 0.6819200000000000D+00*dw(ijkbeg:ijkend,109, 40) &
                          + 0.1409090000000000D+01*dw(ijkbeg:ijkend,118, 40)
      JacC(ijkbeg:ijkend, 29, 40) = + 0.4212500000000000D+00*dw(ijkbeg:ijkend, 89, 40) &
                          + 0.4641300000000000D+00*dw(ijkbeg:ijkend,103, 40) &
                          + 0.6837400000000000D+00*dw(ijkbeg:ijkend,109, 40) &
                          + 0.4303900000000000D+00*dw(ijkbeg:ijkend,118, 40)
      JacC(ijkbeg:ijkend, 30, 40) = + 0.5220000000000000D-01*dw(ijkbeg:ijkend, 89, 40) &
                          + 0.3814000000000000D-01*dw(ijkbeg:ijkend,103, 40) &
                          + 0.6579000000000000D-01*dw(ijkbeg:ijkend,109, 40) &
                          + 0.2051000000000000D-01*dw(ijkbeg:ijkend,118, 40)
      JacC(ijkbeg:ijkend, 40, 40) =  - dw(ijkbeg:ijkend, 89, 40) &
                           - dw(ijkbeg:ijkend, 96, 40) &
                           - dw(ijkbeg:ijkend,103, 40) &
                           - dw(ijkbeg:ijkend,109, 40) &
                           - dw(ijkbeg:ijkend,118, 40)
      JacC(ijkbeg:ijkend, 40,  3) =  - dw(ijkbeg:ijkend, 89,  3)
      JacC(ijkbeg:ijkend,  3, 41) =  - dw(ijkbeg:ijkend, 90, 41)
      JacC(ijkbeg:ijkend,  4, 41) = + 0.8470000000000000D+00*dw(ijkbeg:ijkend, 90, 41) &
                           + dw(ijkbeg:ijkend,119, 41)
      JacC(ijkbeg:ijkend, 21, 41) = + 0.8470000000000000D+00*dw(ijkbeg:ijkend, 90, 41) &
                           - dw(ijkbeg:ijkend, 97, 41) &
                           + dw(ijkbeg:ijkend,104, 41) &
                          + 0.5060000000000000D+00*dw(ijkbeg:ijkend,110, 41) &
                           + dw(ijkbeg:ijkend,119, 41)
      JacC(ijkbeg:ijkend, 25, 41) = + 0.3781500000000000D+00*dw(ijkbeg:ijkend, 90, 41) &
                          + 0.4807400000000000D+00*dw(ijkbeg:ijkend,104, 41) &
                          + 0.2446300000000000D+00*dw(ijkbeg:ijkend,110, 41) &
                          + 0.4272900000000000D+00*dw(ijkbeg:ijkend,119, 41)
      JacC(ijkbeg:ijkend, 25,  3) = + 0.3781500000000000D+00*dw(ijkbeg:ijkend, 90,  3)
      JacC(ijkbeg:ijkend, 28, 41) = + 0.6060000000000000D+00*dw(ijkbeg:ijkend, 90, 41) &
                          + 0.1090000000000000D+01*dw(ijkbeg:ijkend,104, 41) &
                          + 0.3400000000000000D+00*dw(ijkbeg:ijkend,110, 41) &
                          + 0.6860000000000001D+00*dw(ijkbeg:ijkend,119, 41)
      JacC(ijkbeg:ijkend, 31, 41) = + 0.4546300000000000D+00*dw(ijkbeg:ijkend, 90, 41) &
                          + 0.5606400000000000D+00*dw(ijkbeg:ijkend,104, 41) &
                          + 0.7859100000000000D+00*dw(ijkbeg:ijkend,110, 41) &
                          + 0.6116000000000000D+00*dw(ijkbeg:ijkend,119, 41)
      JacC(ijkbeg:ijkend, 32, 41) = + 0.1530000000000000D+00*dw(ijkbeg:ijkend, 90, 41)
      JacC(ijkbeg:ijkend, 41, 41) =  - dw(ijkbeg:ijkend, 90, 41) &
                           - dw(ijkbeg:ijkend, 97, 41) &
                           - dw(ijkbeg:ijkend,104, 41) &
                           - dw(ijkbeg:ijkend,110, 41) &
                           - dw(ijkbeg:ijkend,119, 41)
      JacC(ijkbeg:ijkend, 41,  3) =  - dw(ijkbeg:ijkend, 90,  3)
      JacC(ijkbeg:ijkend,  3, 44) =  - dw(ijkbeg:ijkend, 91, 44)
      JacC(ijkbeg:ijkend,  4, 44) = + 0.9511500000000001D+00*dw(ijkbeg:ijkend, 91, 44) &
                           + dw(ijkbeg:ijkend,120, 44)
      JacC(ijkbeg:ijkend, 21, 44) = + 0.9511500000000001D+00*dw(ijkbeg:ijkend, 91, 44) &
                           - dw(ijkbeg:ijkend, 98, 44) &
                          + 0.1027670000000000D+01*dw(ijkbeg:ijkend,105, 44) &
                           + dw(ijkbeg:ijkend,111, 44) &
                           + dw(ijkbeg:ijkend,120, 44)
      JacC(ijkbeg:ijkend, 31, 44) = + 0.2069930000000000D+01*dw(ijkbeg:ijkend, 91, 44) &
                          + 0.1994610000000000D+01*dw(ijkbeg:ijkend,105, 44) &
                          + 0.1994550000000000D+01*dw(ijkbeg:ijkend,111, 44) &
                          + 0.2819040000000000D+01*dw(ijkbeg:ijkend,120, 44)
      JacC(ijkbeg:ijkend, 32, 44) = + 0.4885000000000000D-01*dw(ijkbeg:ijkend, 91, 44)
      JacC(ijkbeg:ijkend, 44, 44) =  - dw(ijkbeg:ijkend, 91, 44) &
                           - dw(ijkbeg:ijkend, 98, 44) &
                           - dw(ijkbeg:ijkend,105, 44) &
                           - dw(ijkbeg:ijkend,111, 44) &
                           - dw(ijkbeg:ijkend,120, 44)
      JacC(ijkbeg:ijkend, 44,  3) =  - dw(ijkbeg:ijkend, 91,  3)
      JacC(ijkbeg:ijkend,  3, 45) =  - dw(ijkbeg:ijkend, 92, 45)
      JacC(ijkbeg:ijkend, 21, 45) = + 0.1233400000000000D+00*dw(ijkbeg:ijkend, 92, 45) &
                           - dw(ijkbeg:ijkend, 99, 45) &
                          + 0.8299800000000001D+00*dw(ijkbeg:ijkend,106, 45) &
                          + 0.4807900000000000D+00*dw(ijkbeg:ijkend,108, 45) &
                          + 0.5007800000000000D+00*dw(ijkbeg:ijkend,109, 45) &
                          + 0.5060000000000000D+00*dw(ijkbeg:ijkend,110, 45) &
                           + dw(ijkbeg:ijkend,111, 45) &
                          + 0.7566000000000001D-01*dw(ijkbeg:ijkend,112, 45) &
                          + 0.7566000000000001D-01*dw(ijkbeg:ijkend,112, 45) &
                          + 0.1759900000000000D+00*dw(ijkbeg:ijkend,113, 45) &
                          + 0.4915000000000000D-01*dw(ijkbeg:ijkend,121, 45)
      JacC(ijkbeg:ijkend, 28, 45) = + 0.5848000000000000D-01*dw(ijkbeg:ijkend, 92, 45) &
                          + 0.9572300000000000D+00*dw(ijkbeg:ijkend,106, 45) &
                          + 0.7600000000000000D-01*dw(ijkbeg:ijkend,108, 45) &
                          + 0.6819200000000000D+00*dw(ijkbeg:ijkend,109, 45) &
                          + 0.3400000000000000D+00*dw(ijkbeg:ijkend,110, 45) &
                          + 0.3432000000000000D-01*dw(ijkbeg:ijkend,112, 45) &
                          + 0.3432000000000000D-01*dw(ijkbeg:ijkend,112, 45) &
                          + 0.1341400000000000D+00*dw(ijkbeg:ijkend,113, 45) &
                          + 0.3175000000000000D-01*dw(ijkbeg:ijkend,121, 45)
      JacC(ijkbeg:ijkend, 29, 45) = + 0.7368000000000000D-01*dw(ijkbeg:ijkend, 92, 45) &
                          + 0.8295000000000000D-01*dw(ijkbeg:ijkend,106, 45) &
                          + 0.7146100000000000D+00*dw(ijkbeg:ijkend,108, 45) &
                          + 0.6837400000000000D+00*dw(ijkbeg:ijkend,109, 45) &
                          + 0.6969000000000000D-01*dw(ijkbeg:ijkend,112, 45) &
                          + 0.6969000000000000D-01*dw(ijkbeg:ijkend,112, 45) &
                          + 0.4212200000000000D+00*dw(ijkbeg:ijkend,113, 45) &
                          + 0.2936000000000000D-01*dw(ijkbeg:ijkend,121, 45)
      JacC(ijkbeg:ijkend, 31, 45) = + 0.8670000000000000D-01*dw(ijkbeg:ijkend, 92, 45) &
                          + 0.1538700000000000D+00*dw(ijkbeg:ijkend,106, 45) &
                          + 0.6954000000000000D-01*dw(ijkbeg:ijkend,108, 45) &
                          + 0.7859100000000000D+00*dw(ijkbeg:ijkend,110, 45) &
                          + 0.1994550000000000D+01*dw(ijkbeg:ijkend,111, 45) &
                          + 0.1077700000000000D+00*dw(ijkbeg:ijkend,112, 45) &
                          + 0.1077700000000000D+00*dw(ijkbeg:ijkend,112, 45) &
                          + 0.3455000000000000D-01*dw(ijkbeg:ijkend,121, 45)
      JacC(ijkbeg:ijkend, 38, 45) = + 0.7813400000000000D+00*dw(ijkbeg:ijkend, 92, 45) &
                          - 0.4396900000000000D+00*dw(ijkbeg:ijkend,106, 45) &
                          + 0.5148000000000000D+00*dw(ijkbeg:ijkend,108, 45) &
                          + 0.5007800000000000D+00*dw(ijkbeg:ijkend,109, 45) &
                          + 0.5060000000000000D+00*dw(ijkbeg:ijkend,110, 45) &
                           + dw(ijkbeg:ijkend,111, 45) &
                          + 0.1667020000000000D+01*dw(ijkbeg:ijkend,112, 45) &
                          + 0.1667020000000000D+01*dw(ijkbeg:ijkend,112, 45) &
                          + 0.5103700000000000D+00*dw(ijkbeg:ijkend,113, 45) &
                          + 0.9191000000000000D+00*dw(ijkbeg:ijkend,121, 45) &
                           + dw(ijkbeg:ijkend,125, 45)
      JacC(ijkbeg:ijkend, 45,  3) = - 0.9046800000000000D+00*dw(ijkbeg:ijkend, 92,  3)
      JacC(ijkbeg:ijkend, 47, 45) = + 0.2563000000000000D-01*dw(ijkbeg:ijkend, 92, 45) &
                          + 0.2212000000000000D-01*dw(ijkbeg:ijkend,106, 45) &
                          + 0.1130600000000000D+00*dw(ijkbeg:ijkend,108, 45) &
                          + 0.1593000000000000D-01*dw(ijkbeg:ijkend,112, 45) &
                          + 0.1593000000000000D-01*dw(ijkbeg:ijkend,112, 45) &
                          + 0.1021000000000000D-01*dw(ijkbeg:ijkend,121, 45) &
                           - dw(ijkbeg:ijkend,125, 45)
      JacC(ijkbeg:ijkend,  3, 46) =  - dw(ijkbeg:ijkend, 93, 46)
      JacC(ijkbeg:ijkend,  4, 46) = + 0.1815990000000000D+01*dw(ijkbeg:ijkend, 93, 46) &
                          + 0.3244000000000000D+00*dw(ijkbeg:ijkend,107, 46) &
                          + 0.7500000000000000D+00*dw(ijkbeg:ijkend,115, 46) &
                          + 0.7500000000000000D+00*dw(ijkbeg:ijkend,115, 46) &
                          + 0.1740720000000000D+01*dw(ijkbeg:ijkend,122, 46)
      JacC(ijkbeg:ijkend, 21, 46) = + 0.1840100000000000D+00*dw(ijkbeg:ijkend, 93, 46) &
                           - dw(ijkbeg:ijkend,100, 46) &
                          + 0.6756000000000000D+00*dw(ijkbeg:ijkend,107, 46) &
                          + 0.1759900000000000D+00*dw(ijkbeg:ijkend,113, 46) &
                           + dw(ijkbeg:ijkend,114, 46) &
                           + dw(ijkbeg:ijkend,114, 46) &
                          + 0.5000000000000000D+00*dw(ijkbeg:ijkend,115, 46) &
                          + 0.5000000000000000D+00*dw(ijkbeg:ijkend,115, 46) &
                          + 0.2592800000000000D+00*dw(ijkbeg:ijkend,122, 46)
      JacC(ijkbeg:ijkend, 28, 46) = + 0.2341900000000000D+00*dw(ijkbeg:ijkend, 93, 46) &
                          + 0.8862500000000000D+00*dw(ijkbeg:ijkend,107, 46) &
                          + 0.1341400000000000D+00*dw(ijkbeg:ijkend,113, 46) &
                          + 0.3530000000000000D+00*dw(ijkbeg:ijkend,115, 46) &
                          + 0.3530000000000000D+00*dw(ijkbeg:ijkend,115, 46) &
                          + 0.2074000000000000D+00*dw(ijkbeg:ijkend,122, 46)
      JacC(ijkbeg:ijkend, 29, 46) = + 0.1011820000000000D+01*dw(ijkbeg:ijkend, 93, 46) &
                          + 0.4152400000000000D+00*dw(ijkbeg:ijkend,107, 46) &
                          + 0.4212200000000000D+00*dw(ijkbeg:ijkend,113, 46) &
                          + 0.9250000000000000D+00*dw(ijkbeg:ijkend,115, 46) &
                          + 0.9250000000000000D+00*dw(ijkbeg:ijkend,115, 46) &
                          + 0.9185000000000000D+00*dw(ijkbeg:ijkend,122, 46)
      JacC(ijkbeg:ijkend, 30, 46) = + 0.3786200000000000D+00*dw(ijkbeg:ijkend, 93, 46) &
                          + 0.9667000000000001D-01*dw(ijkbeg:ijkend,107, 46) &
                          + 0.1082200000000000D+00*dw(ijkbeg:ijkend,113, 46) &
                          + 0.2170000000000000D+00*dw(ijkbeg:ijkend,115, 46) &
                          + 0.2170000000000000D+00*dw(ijkbeg:ijkend,115, 46) &
                          + 0.3474000000000000D+00*dw(ijkbeg:ijkend,122, 46)
      JacC(ijkbeg:ijkend, 32, 46) = + 0.1840100000000000D+00*dw(ijkbeg:ijkend, 93, 46) &
                           + dw(ijkbeg:ijkend,100, 46) &
                          + 0.6756000000000000D+00*dw(ijkbeg:ijkend,107, 46) &
                          + 0.6656200000000000D+00*dw(ijkbeg:ijkend,113, 46) &
                          + 0.2000000000000000D+01*dw(ijkbeg:ijkend,114, 46) &
                          + 0.2000000000000000D+01*dw(ijkbeg:ijkend,114, 46) &
                          + 0.1250000000000000D+01*dw(ijkbeg:ijkend,115, 46) &
                          + 0.1250000000000000D+01*dw(ijkbeg:ijkend,115, 46) &
                          + 0.2592800000000000D+00*dw(ijkbeg:ijkend,122, 46)
      JacC(ijkbeg:ijkend, 46, 46) =  - dw(ijkbeg:ijkend, 93, 46) &
                           - dw(ijkbeg:ijkend,100, 46) &
                           - dw(ijkbeg:ijkend,107, 46) &
                           - dw(ijkbeg:ijkend,113, 46) &
                          - 0.2000000000000000D+01*dw(ijkbeg:ijkend,114, 46) &
                          - 0.2000000000000000D+01*dw(ijkbeg:ijkend,114, 46) &
                          - 0.2000000000000000D+01*dw(ijkbeg:ijkend,115, 46) &
                          - 0.2000000000000000D+01*dw(ijkbeg:ijkend,115, 46) &
                           - dw(ijkbeg:ijkend,122, 46)
      JacC(ijkbeg:ijkend, 46,  3) =  - dw(ijkbeg:ijkend, 93,  3)
      JacC(ijkbeg:ijkend, 34, 38) =  + dw(ijkbeg:ijkend, 94, 38)
      JacC(ijkbeg:ijkend, 34, 21) =  + dw(ijkbeg:ijkend, 94, 21)
      JacC(ijkbeg:ijkend, 38, 21) =  - dw(ijkbeg:ijkend, 94, 21)
      JacC(ijkbeg:ijkend, 35, 39) = + 0.1005240000000000D+01*dw(ijkbeg:ijkend, 95, 39)
      JacC(ijkbeg:ijkend, 35, 21) = + 0.1005240000000000D+01*dw(ijkbeg:ijkend, 95, 21) &
                          + 0.1005240000000000D+01*dw(ijkbeg:ijkend, 96, 21) &
                          + 0.1005240000000000D+01*dw(ijkbeg:ijkend, 97, 21) &
                          + 0.1005240000000000D+01*dw(ijkbeg:ijkend, 98, 21) &
                          + 0.8090400000000000D+00*dw(ijkbeg:ijkend, 99, 21) &
                          + 0.1005240000000000D+01*dw(ijkbeg:ijkend,123, 21)
      JacC(ijkbeg:ijkend, 39, 21) =  - dw(ijkbeg:ijkend, 95, 21)
      JacC(ijkbeg:ijkend, 35, 40) = + 0.1005240000000000D+01*dw(ijkbeg:ijkend, 96, 40)
      JacC(ijkbeg:ijkend, 40, 21) =  - dw(ijkbeg:ijkend, 96, 21)
      JacC(ijkbeg:ijkend, 35, 41) = + 0.1005240000000000D+01*dw(ijkbeg:ijkend, 97, 41)
      JacC(ijkbeg:ijkend, 41, 21) =  - dw(ijkbeg:ijkend, 97, 21)
      JacC(ijkbeg:ijkend, 35, 44) = + 0.1005240000000000D+01*dw(ijkbeg:ijkend, 98, 44)
      JacC(ijkbeg:ijkend, 44, 21) =  - dw(ijkbeg:ijkend, 98, 21)
      JacC(ijkbeg:ijkend,  1, 45) = + 0.1730700000000000D+00*dw(ijkbeg:ijkend, 99, 45)
      JacC(ijkbeg:ijkend, 35, 45) = + 0.8090400000000000D+00*dw(ijkbeg:ijkend, 99, 45)
      JacC(ijkbeg:ijkend, 37, 45) = + 0.1730700000000000D+00*dw(ijkbeg:ijkend, 99, 45) &
                          + 0.1368400000000000D+00*dw(ijkbeg:ijkend,106, 45) &
                          + 0.4981000000000000D+00*dw(ijkbeg:ijkend,108, 45) &
                          + 0.4992200000000000D+00*dw(ijkbeg:ijkend,109, 45) &
                          + 0.4940000000000000D+00*dw(ijkbeg:ijkend,110, 45) &
                          + 0.9955000000000000D-01*dw(ijkbeg:ijkend,112, 45) &
                          + 0.9955000000000000D-01*dw(ijkbeg:ijkend,112, 45) &
                          + 0.4896300000000000D+00*dw(ijkbeg:ijkend,113, 45)
      JacC(ijkbeg:ijkend, 37, 21) = + 0.1730700000000000D+00*dw(ijkbeg:ijkend, 99, 21)
      JacC(ijkbeg:ijkend, 45, 21) =  - dw(ijkbeg:ijkend, 99, 21)
      JacC(ijkbeg:ijkend, 32, 21) =  + dw(ijkbeg:ijkend,100, 21)
      JacC(ijkbeg:ijkend, 46, 21) =  - dw(ijkbeg:ijkend,100, 21)
      JacC(ijkbeg:ijkend, 29, 38) = + 0.5607000000000000D+00*dw(ijkbeg:ijkend,102, 38) &
                          + 0.4641300000000000D+00*dw(ijkbeg:ijkend,103, 38) &
                          + 0.8295000000000000D-01*dw(ijkbeg:ijkend,106, 38) &
                          + 0.4152400000000000D+00*dw(ijkbeg:ijkend,107, 38)
      JacC(ijkbeg:ijkend, 30, 38) = + 0.9673000000000000D-01*dw(ijkbeg:ijkend,102, 38) &
                          + 0.3814000000000000D-01*dw(ijkbeg:ijkend,103, 38) &
                          + 0.9667000000000001D-01*dw(ijkbeg:ijkend,107, 38)
      JacC(ijkbeg:ijkend, 31, 38) = + 0.7976000000000000D-01*dw(ijkbeg:ijkend,102, 38) &
                          + 0.5606400000000000D+00*dw(ijkbeg:ijkend,104, 38) &
                          + 0.1994610000000000D+01*dw(ijkbeg:ijkend,105, 38) &
                          + 0.1538700000000000D+00*dw(ijkbeg:ijkend,106, 38)
      JacC(ijkbeg:ijkend, 39, 38) = - 0.9961500000000000D+00*dw(ijkbeg:ijkend,102, 38)
      JacC(ijkbeg:ijkend, 47, 38) = + 0.1337000000000000D+00*dw(ijkbeg:ijkend,102, 38) &
                          + 0.2212000000000000D-01*dw(ijkbeg:ijkend,106, 38) &
                           - dw(ijkbeg:ijkend,124, 38)
      JacC(ijkbeg:ijkend, 38, 40) =  - dw(ijkbeg:ijkend,103, 40) &
                          + 0.5007800000000000D+00*dw(ijkbeg:ijkend,109, 40)
      JacC(ijkbeg:ijkend, 40, 38) =  - dw(ijkbeg:ijkend,103, 38)
      JacC(ijkbeg:ijkend, 25, 38) = + 0.4807400000000000D+00*dw(ijkbeg:ijkend,104, 38)
      JacC(ijkbeg:ijkend, 38, 41) =  - dw(ijkbeg:ijkend,104, 41) &
                          + 0.5060000000000000D+00*dw(ijkbeg:ijkend,110, 41)
      JacC(ijkbeg:ijkend, 41, 38) =  - dw(ijkbeg:ijkend,104, 38)
      JacC(ijkbeg:ijkend, 28, 44) =  + dw(ijkbeg:ijkend,105, 44)
      JacC(ijkbeg:ijkend, 38, 44) =  - dw(ijkbeg:ijkend,105, 44) &
                           + dw(ijkbeg:ijkend,111, 44)
      JacC(ijkbeg:ijkend, 44, 38) =  - dw(ijkbeg:ijkend,105, 38)
      JacC(ijkbeg:ijkend, 37, 38) = + 0.1368400000000000D+00*dw(ijkbeg:ijkend,106, 38)
      JacC(ijkbeg:ijkend, 45, 38) = - 0.9404600000000000D+00*dw(ijkbeg:ijkend,106, 38)
      JacC(ijkbeg:ijkend, 32, 38) = + 0.6756000000000000D+00*dw(ijkbeg:ijkend,107, 38)
      JacC(ijkbeg:ijkend, 38, 46) =  - dw(ijkbeg:ijkend,107, 46) &
                          + 0.5103700000000000D+00*dw(ijkbeg:ijkend,113, 46)
      JacC(ijkbeg:ijkend, 46, 38) =  - dw(ijkbeg:ijkend,107, 38)
      JacC(ijkbeg:ijkend, 30, 45) = + 0.1881900000000000D+00*dw(ijkbeg:ijkend,108, 45) &
                          + 0.6579000000000000D-01*dw(ijkbeg:ijkend,109, 45) &
                          + 0.2190000000000000D-01*dw(ijkbeg:ijkend,112, 45) &
                          + 0.2190000000000000D-01*dw(ijkbeg:ijkend,112, 45) &
                          + 0.1082200000000000D+00*dw(ijkbeg:ijkend,113, 45)
      JacC(ijkbeg:ijkend, 37, 39) = + 0.4981000000000000D+00*dw(ijkbeg:ijkend,108, 39)
      JacC(ijkbeg:ijkend, 39, 45) = - 0.9917200000000000D+00*dw(ijkbeg:ijkend,108, 45)
      JacC(ijkbeg:ijkend, 45, 39) =  - dw(ijkbeg:ijkend,108, 39)
      JacC(ijkbeg:ijkend, 37, 40) = + 0.4992200000000000D+00*dw(ijkbeg:ijkend,109, 40)
      JacC(ijkbeg:ijkend, 40, 45) =  - dw(ijkbeg:ijkend,109, 45)
      JacC(ijkbeg:ijkend, 45, 40) =  - dw(ijkbeg:ijkend,109, 40)
      JacC(ijkbeg:ijkend, 25, 45) = + 0.2446300000000000D+00*dw(ijkbeg:ijkend,110, 45)
      JacC(ijkbeg:ijkend, 37, 41) = + 0.4940000000000000D+00*dw(ijkbeg:ijkend,110, 41)
      JacC(ijkbeg:ijkend, 41, 45) =  - dw(ijkbeg:ijkend,110, 45)
      JacC(ijkbeg:ijkend, 45, 41) =  - dw(ijkbeg:ijkend,110, 41)
      JacC(ijkbeg:ijkend, 44, 45) =  - dw(ijkbeg:ijkend,111, 45)
      JacC(ijkbeg:ijkend, 45, 44) =  - dw(ijkbeg:ijkend,111, 44)
      JacC(ijkbeg:ijkend, 32, 45) = + 0.6656200000000000D+00*dw(ijkbeg:ijkend,113, 45)
      JacC(ijkbeg:ijkend, 37, 46) = + 0.4896300000000000D+00*dw(ijkbeg:ijkend,113, 46)
      JacC(ijkbeg:ijkend, 45, 46) =  - dw(ijkbeg:ijkend,113, 46)
      JacC(ijkbeg:ijkend, 46, 45) =  - dw(ijkbeg:ijkend,113, 45)
      JacC(ijkbeg:ijkend,  5, 38) =  - dw(ijkbeg:ijkend,116, 38)
      JacC(ijkbeg:ijkend, 38,  5) =  - dw(ijkbeg:ijkend,116,  5) &
                          + 0.9730999999999999D-01*dw(ijkbeg:ijkend,117,  5) &
                          + 0.9191000000000000D+00*dw(ijkbeg:ijkend,121,  5)
      JacC(ijkbeg:ijkend,  5, 39) =  - dw(ijkbeg:ijkend,117, 39)
      JacC(ijkbeg:ijkend, 39,  5) = - 0.9100600000000000D+00*dw(ijkbeg:ijkend,117,  5)
      JacC(ijkbeg:ijkend,  5, 40) =  - dw(ijkbeg:ijkend,118, 40)
      JacC(ijkbeg:ijkend, 40,  5) =  - dw(ijkbeg:ijkend,118,  5)
      JacC(ijkbeg:ijkend,  5, 41) =  - dw(ijkbeg:ijkend,119, 41)
      JacC(ijkbeg:ijkend, 41,  5) =  - dw(ijkbeg:ijkend,119,  5)
      JacC(ijkbeg:ijkend,  5, 44) =  - dw(ijkbeg:ijkend,120, 44)
      JacC(ijkbeg:ijkend, 44,  5) =  - dw(ijkbeg:ijkend,120,  5)
      JacC(ijkbeg:ijkend,  5, 45) =  - dw(ijkbeg:ijkend,121, 45)
      JacC(ijkbeg:ijkend,  5, 46) =  - dw(ijkbeg:ijkend,122, 46)
      JacC(ijkbeg:ijkend, 21, 47) =  - dw(ijkbeg:ijkend,123, 47) &
                           + dw(ijkbeg:ijkend,124, 47)
      JacC(ijkbeg:ijkend, 35, 47) = + 0.1005240000000000D+01*dw(ijkbeg:ijkend,123, 47)
      JacC(ijkbeg:ijkend, 47, 47) =  - dw(ijkbeg:ijkend,123, 47) &
                           - dw(ijkbeg:ijkend,124, 47) &
                           - dw(ijkbeg:ijkend,125, 47) &
                          - 0.2000000000000000D+01*dw(ijkbeg:ijkend,126, 47) &
                          - 0.2000000000000000D+01*dw(ijkbeg:ijkend,126, 47) &
                           - dw(ijkbeg:ijkend,127, 47) &
                           - dw(ijkbeg:ijkend,128, 47)
      JacC(ijkbeg:ijkend, 47, 21) =  - dw(ijkbeg:ijkend,123, 21)
      JacC(ijkbeg:ijkend, 28, 47) =  + dw(ijkbeg:ijkend,124, 47)
      JacC(ijkbeg:ijkend, 38, 47) =  - dw(ijkbeg:ijkend,124, 47) &
                           + dw(ijkbeg:ijkend,125, 47)
      JacC(ijkbeg:ijkend, 45, 47) =  - dw(ijkbeg:ijkend,125, 47)
      JacC(ijkbeg:ijkend,  3, 47) =  - dw(ijkbeg:ijkend,127, 47)
      JacC(ijkbeg:ijkend,  4, 47) =  + dw(ijkbeg:ijkend,127, 47) &
                           + dw(ijkbeg:ijkend,128, 47)
      JacC(ijkbeg:ijkend,  5, 47) =  - dw(ijkbeg:ijkend,128, 47)
			!END DO

   END SUBROUTINE jacdchemdc

  END MODULE mod_chem_spack_jacdchemdc
